@@ -14,23 +14,22 @@
 ;;; 0 code-component-ish
 ;;; 1 an offset relative to the start of the code-component or an
 ;;;   absolute address
-(defconstant +elements-per-sample+ 2)
+(defconstant +elements-per-pc-loc+ 2)
 
 ;;; 0 the trace start marker (trace-start . END-INDEX)
 ;;; 1 the current thread, an SB-THREAD:THREAD instance
-;;; 2 the current internal real time
-(defconstant +elements-per-trace-start+ 3)
+(defconstant +elements-per-trace-start+ 2)
 
 (declaim (inline make-sample-vector))
 (defun make-sample-vector (max-samples)
   (make-array (* max-samples
-                 ;; Arbitrary guess at how many samples we'll be
+                 ;; Arbitrary guess at how many pc-locs we'll be
                  ;; taking for each trace. The exact amount doesn't
                  ;; matter, this is just to decrease the amount of
                  ;; re-allocation that will need to be done.
                  10
-                 ;; Each sample takes two cells in the vector
-                 +elements-per-sample+)))
+                 ;; Each pc-loc takes two cells in the vector
+                 +elements-per-pc-loc+)))
 
 ;;; Encapsulate all the information about a sampling run
 (defstruct (samples
@@ -78,7 +77,7 @@
                           approximately ~:D sample~:P), doubling the ~
                           size~%"
           (samples-trace-count samples)
-          (truncate (samples-index samples) +elements-per-sample+)))
+          (truncate (samples-index samples) +elements-per-pc-loc+)))
 
 (declaim (inline ensure-samples-vector))
 (defun ensure-samples-vector (samples)
@@ -89,7 +88,7 @@
     ;; accommodate the largest chunk of elements that we could
     ;; potentially store in one go (currently 3 elements, stored by
     ;; RECORD-TRACE-START).
-    (values (if (< (length vector) (+ index (max +elements-per-sample+
+    (values (if (< (length vector) (+ index (max +elements-per-pc-loc+
                                                  +elements-per-trace-start+)))
                 (let ((new-vector (make-array (* 2 index))))
                   (note-sample-vector-full samples)
@@ -103,8 +102,7 @@
   (multiple-value-bind (vector index) (ensure-samples-vector samples)
     (let ((trace-start (cons 'trace-start nil)))
       (setf (aref vector index)       trace-start
-            (aref vector (+ index 1)) sb-thread:*current-thread*
-            (aref vector (+ index 2)) (get-internal-real-time))
+            (aref vector (+ index 1)) sb-thread:*current-thread*)
       (setf (samples-index samples)       (+ index +elements-per-trace-start+)
             (samples-trace-start samples) trace-start))))
 
@@ -118,56 +116,46 @@
     ;; adjacent cells.
     (setf (aref vector index) info
           (aref vector (1+ index)) pc-or-offset)
-    (setf (samples-index samples) (+ index +elements-per-sample+))))
+    (setf (samples-index samples) (+ index +elements-per-pc-loc+))))
 
 ;;; Trace and sample and access functions
 
 (defun map-traces (function samples)
   "Call FUNCTION on each trace in SAMPLES
 
-The lambda list of FUNCTION has to be compatible to
+The signature of FUNCTION must be compatible with (thread trace).
 
-  (thread time trace)
-
-. FUNCTION is called once for each trace such that THREAD is the
-SB-THREAD:TREAD instance that was sampled to produce TRACE, TIME is
-the internal real time at which TRACE was produced and TRACE is an
-opaque object whose only purpose is being used as the second argument
-to MAP-TRACE-SAMPLES.
+FUNCTION is called once for each trace where THREAD is the SB-THREAD:TREAD
+instance which was sampled to produce TRACE, and TRACE is an opaque object
+to be passed to MAP-TRACE-SAMPLES.
 
 EXPERIMENTAL: Interface subject to change."
   (let ((function (sb-kernel:%coerce-callable-to-fun function))
         (vector (samples-vector samples))
-        (index (samples-index samples))
-        (start-time (samples-start-time samples)))
+        (index (samples-index samples)))
     (when (plusp index)
       (sb-int:aver (typep (aref vector 0) '(cons (eql trace-start) index)))
       (loop for start = 0 then end
             while (< start index)
             for end = (cdr (aref vector start))
             for thread = (aref vector (+ start 1))
-            for time = (/ (- (aref vector (+ start 2)) start-time)
-                          internal-time-units-per-second)
             do (let ((trace (list vector start end)))
-                 (funcall function thread time trace))))))
+                 (funcall function thread trace))))))
 
+;;; FIXME: rename to MAP-TRACE-PC-LOCS,
+;;; and also maybe rename SAMPLE-PC to PC-LOC-PC.
 (defun map-trace-samples (function trace)
   "Call FUNCTION on each sample in TRACE.
 
-The lambda list of FUNCTION has to be compatible to
-
-  (info pc-or-offset)
-
-.
-
-TRACE is an object as received by a function passed to MAP-TRACES.
+The signature of FUNCTION must be compatible with (info pc-or-offset).
+TRACE is an object as received by the function passed to MAP-TRACES.
 
 EXPERIMENTAL: Interface subject to change."
   (let ((function (sb-kernel:%coerce-callable-to-fun function)))
     (destructuring-bind (samples start end) trace
-      (loop for i from (- end +elements-per-sample+)
+      (loop for i from (- end +elements-per-pc-loc+)
             downto (+ start +elements-per-trace-start+ -1)
-            by +elements-per-sample+
+            by +elements-per-pc-loc+
             for info = (aref samples i)
             for pc-or-offset = (aref samples (1+ i))
             do (funcall function info pc-or-offset)))))
@@ -176,17 +164,13 @@ EXPERIMENTAL: Interface subject to change."
 (defun map-all-samples (function &optional (samples *samples*))
   "Call FUNCTION on each sample in SAMPLES.
 
-The lambda list of FUNCTION has to be compatible to
-
-  (info pc-or-offset)
-
-.
+The signature of FUNCTION must be compatible with (info pc-or-offset).
 
 SAMPLES is usually the value of *SAMPLES* after a profiling run.
 
 EXPERIMENTAL: Interface subject to change."
-  (sb-int:dx-flet ((do-trace (thread time trace)
-                     (declare (ignore thread time))
+  (sb-int:dx-flet ((do-trace (thread trace)
+                     (declare (ignore thread))
                      (map-trace-samples function trace)))
     (map-traces #'do-trace samples)))
 
@@ -200,7 +184,7 @@ EXPERIMENTAL: Interface subject to change."
   (etypecase info
     ;; Assembly routines or foreign functions don't move around, so
     ;; we've stored a raw PC
-    ((or null sb-kernel:code-component string)
+    ((or null sb-kernel:code-component string symbol)
      pc-or-offset)
     ;; Lisp functions might move, so we've stored a offset from the
     ;; start of the code component.
@@ -221,18 +205,9 @@ EXPERIMENTAL: Interface subject to change."
 
 (defvar *show-progress* nil)
 
-(defvar *old-sampling* nil)
-
 ;; Call count encapsulation information
 (defvar *encapsulations* (make-hash-table :test 'equal))
 
-(defun turn-off-sampling ()
-  (setq *old-sampling* *sampling*)
-  (setq *sampling* nil))
-
-(defun turn-on-sampling ()
-  (setq *sampling* *old-sampling*))
-
 (defun show-progress (format-string &rest args)
   (when *show-progress*
     (apply #'format t format-string args)
@@ -257,14 +232,17 @@ EXPERIMENTAL: Interface subject to change."
 (defun debug-info (pc)
   (declare (type system-area-pointer pc)
            (muffle-conditions compiler-note))
-  (let ((code (sb-di::code-header-from-pc pc)))
+  (let ((code (sb-di::code-header-from-pc pc))
+        (pc-int (sap-int pc)))
     (cond ((not code)
            (let ((name (sap-foreign-symbol pc)))
              (if name
                  (values (format nil "foreign function ~a" name)
-                         (sap-int pc)
-                         :foreign)
-                 (values nil (sap-int pc) :foreign))))
+                         pc-int :foreign)
+                 (values nil pc-int :foreign))))
+          ((eq code sb-fasl:*assembler-routines*)
+           (values (sb-disassem::find-assembler-routine pc-int)
+                   pc-int :asm-routine))
           (t
            (let* ((code-header-len (* (sb-kernel:code-header-words code)
                                       sb-vm:n-word-bytes))
@@ -277,15 +255,15 @@ EXPERIMENTAL: Interface subject to change."
                   (di (unless (typep (sb-kernel:%code-debug-info code)
                                      'sb-c::compiled-debug-info)
                         (return-from debug-info
-                          (values code (sap-int pc)))))
-                  (pc-offset (- (sap-int pc)
+                          (values code pc-int nil))))
+                  (pc-offset (- pc-int
                                 (- (sb-kernel:get-lisp-obj-address code)
                                    sb-vm:other-pointer-lowtag)
                                 code-header-len))
                   (df (sb-di::debug-fun-from-pc code pc-offset)))
              #+immobile-code (declare (ignorable di))
              (cond ((typep df 'sb-di::bogus-debug-fun)
-                    (values code (sap-int pc) nil))
+                    (values code pc-int nil))
                    (df
                     ;; The code component might be moved by the GC. Store
                     ;; a PC offset, and reconstruct the data in
@@ -339,15 +317,17 @@ EXPERIMENTAL: Interface subject to change."
       ;; Don't flood the system with more interrupts if the last
       ;; set is still being delivered.
       (unless (sb-thread:mutex-owner lock)
-        (sb-thread::with-system-mutex (lock)
+        (with-system-mutex (lock)
           (dolist (thread (profiled-threads))
-            (sb-thread:with-os-thread (os-thread thread)
-              (sb-thread::pthread-kill os-thread sb-unix:sigprof))))))
+            (sb-thread:with-deathlok (thread c-thread)
+              (unless (= c-thread 0)
+                (sb-thread:pthread-kill (sb-thread::thread-os-thread thread)
+                                        sb-unix:sigprof)))))))
     #-sb-thread
     (unix-kill 0 sb-unix:sigprof))
 
   (defun sigprof-handler (signal code scp)
-    (declare (ignore signal code) (optimize speed (space 0))
+    (declare (ignore signal code) (optimize speed)
              (disable-package-locks sb-di::x86-call-context)
              (muffle-conditions compiler-note)
              (type system-area-pointer scp))
@@ -368,7 +348,8 @@ EXPERIMENTAL: Interface subject to change."
                  ;; pointless there -- though it may be that our mach magic is
                  ;; partially to blame?
                  (or (not (eq :cpu profiling)) (profiled-thread-p self)))
-        (sb-thread::with-system-mutex (*profiler-lock* :without-gcing t)
+        (with-code-pages-pinned (:dynamic)
+         (with-system-mutex (*profiler-lock*)
           (let ((samples *samples*)
                 ;; Don't touch the circularity hash-table
                 *print-circle*)
@@ -377,7 +358,9 @@ EXPERIMENTAL: Interface subject to change."
                           (samples-max-samples samples)))
               (with-alien ((scp (* os-context-t) :local scp))
                 (let* ((pc-ptr (sb-vm:context-pc scp))
-                       (fp (sb-vm::context-register scp #.sb-vm::ebp-offset)))
+                       (fp (sb-vm::context-register scp
+                                                    #+x86-64 #.sb-vm::rbp-offset
+                                                    #+x86 #.sb-vm::ebp-offset)))
                   ;; foreign code might not have a useful frame
                   ;; pointer in ebp/rbp, so make sure it looks
                   ;; reasonable before walking the stack
@@ -415,13 +398,13 @@ EXPERIMENTAL: Interface subject to change."
                         (when (zerop i)
                           (decf (samples-index samples)
                                 (+ +elements-per-trace-start+
-                                   (* +elements-per-sample+ (1+ i)))))
+                                   (* +elements-per-pc-loc+ (1+ i)))))
                         (return)))
                     (record-trace-end samples))))
               ;; Reset thread-local allocation counter before interrupts
               ;; are enabled.
               (when (eq t sb-vm::*alloc-signal*)
-                (setf sb-vm:*alloc-signal* (1- (samples-alloc-interval samples)))))))))
+                (setf sb-vm:*alloc-signal* (1- (samples-alloc-interval samples))))))))))
     nil))
 
 ;; FIXME: On non-x86 platforms we don't yet walk the call stack deeper
