@@ -465,11 +465,19 @@ information."
                      (< a (get-lisp-obj-address sb-vm:*control-stack-end*)))
                 sb-thread:*current-thread*)
                (all-threads
-                ;; find a stack whose base is nearest and below A.
-                (awhen (sb-thread::avl-find<= a sb-thread::*all-threads*)
-                  (let ((thread (sb-thread::avlnode-data it)))
-                    (when (< a (sb-thread::thread-stack-end thread))
-                      thread))))))))
+                (macrolet ((in-stack-range-p ()
+                             `(and (>= a (sb-thread::thread-control-stack-start thread))
+                                   (< a (sb-thread::thread-control-stack-end thread)))))
+                  #+win32 ; exhaustive search
+                  (dolist (thread (sb-thread:list-all-threads)) ; conses, but I don't care
+                    (when (in-stack-range-p)
+                      (return thread)))
+                  #-win32
+                  ;; find a stack whose primitive-thread is nearest and above A.
+                  (awhen (sb-thread::avl-find>= a sb-thread::*all-threads*)
+                    (let ((thread (sb-thread::avlnode-data it)))
+                      (when (in-stack-range-p)
+                        thread)))))))))
 
 ;;;; frame printing
 
@@ -1636,7 +1644,7 @@ forms that explicitly control this kind of evaluation.")
   (show-restarts *debug-restarts* *debug-io*))
 
 (!def-debug-command "BACKTRACE" ()
- (print-backtrace :count (read-if-available sb-xc:most-positive-fixnum)))
+ (print-backtrace :count (read-if-available most-positive-fixnum)))
 
 (!def-debug-command "PRINT" ()
   (print-frame-call *current-frame* *debug-io*))
@@ -1933,3 +1941,39 @@ forms that explicitly control this kind of evaluation.")
   (if (listen-skip-whitespace *debug-io*)
       (read *debug-io*)
       default))
+
+#+(and sb-devel x86-64)
+(defun show-catch-tags ()
+  (declare (notinline format))
+  (let ((sap (descriptor-sap sb-vm:*current-catch-block*)))
+    (loop
+     (let ((tag (sap-ref-lispobj sap (ash sb-vm:catch-block-tag-slot 3)))
+           (link (sap-ref-sap sap (ash sb-vm:catch-block-previous-catch-slot 3))))
+       (format t "~S ~A~%" tag link)
+       (setq sap link)
+       (if (= (sap-int sap) 0) (return))))))
+
+;;; Sometimes in cold-init it is not possible to call LIST-BACKTRACE
+;;; because that depends on a zillion things being set up correctly.
+;;; This simple version seems to always work.
+#+(or x86 x86-64)
+(defun ultralite-backtrace (&optional (decode-pcs t))
+  ;; this misses the current frame but that's perfectly fine for its intended use
+  (let ((fp (current-fp)) (list))
+    (loop
+     (let ((prev-fp (sap-ref-sap fp 0)))
+       (cond ((and (sap> prev-fp fp)
+                   (sap< prev-fp (descriptor-sap sb-vm:*control-stack-end*)))
+              (push (sap-ref-sap fp sb-vm:n-word-bytes) list) ; pc
+              (setq fp prev-fp))
+             (t
+              (return)))))
+    (setq list (nreverse list))
+    (if decode-pcs
+        (mapcar (lambda (pc)
+                  (let ((code (sb-di::code-header-from-pc pc)))
+                    (if code
+                        (cons code (sap- pc (code-instructions code)))
+                        pc)))
+                list)
+        list)))
