@@ -264,69 +264,70 @@ statistics are appended to it."
   ;; B) interrupts are disabled somewhere up the call chain since we
   ;;    don't want to run user code in such a case.
   ;;
+  ;; Condition (A) is tested in the C runtime at can_invoke_post_gc().
+  ;; Condition (B) is tested here.
   ;; The long-term solution will be to keep a separate thread for
   ;; after-gc hooks.
   ;; Finalizers are in a separate thread (usually),
   ;; but it's not permissible to invoke CONDITION-NOTIFY from a
   ;; dying thread, so we still need the guard for that, but not
   ;; the guard for whether interupts are enabled.
-  (when (and
-         #+sb-thread (/= 0 (sap-int (sb-vm::current-thread-offset-sap
-                                     sb-vm::thread-lisp-thread-slot)))
-         (sb-thread:thread-alive-p sb-thread:*current-thread*))
-    #+sb-thread (alien-funcall (extern-alien "empty_thread_recyclebin" (function void)))
-    (let ((threadp #+sb-thread (%instancep sb-impl::*finalizer-thread*)))
-      (when threadp
+  #+sb-thread (alien-funcall (extern-alien "empty_thread_recyclebin" (function void)))
+  (let ((threadp #+sb-thread (%instancep sb-impl::*finalizer-thread*)))
+    (when threadp
+      ;; grabbing a lock, need to clear out any waiting-for to avoid
+      ;; spurios deadlock detections.
+      (sb-thread::without-thread-waiting-for ()
         ;; It's OK to frob a condition variable regardless of
         ;; *allow-with-interrupts*, and probably OK to start a thread.
         ;; For consistency with the previous behavior, we delay finalization
         ;; if there is no finalizer thread and interrupts are disabled.
         ;; That's my excuse anyway, not having looked more in-depth.
-        (run-pending-finalizers))
-      ;; Here's one reason that MAX_INTERRUPTS is as high as it is.
-      ;; If the thread that performed GC runs post-GC hooks which cons enough to
-      ;; cause another GC while in the hooks, then as soon as interrupts are allowed
-      ;; again, a GC can be invoked "recursively" - while there is a maybe_gc() on
-      ;; the C call stack. It's not even true that _this_ thread had to cons a ton -
-      ;; any thread could have, causing this thread to hit the GC trigger which sets the
-      ;; P-A interrupted bit which causes the interrupt instruction at the end of a
-      ;; pseudo-atomic sequence to take a signal hit. So with interrupts eanbled,
-      ;; we get back into the GC, which calls post-GC, which might cons ...
-      ;; See the example at the bottom of src/code/final for a clear picture.
-      ;;
-      ;; To mitigate that problem, if you have no hooks, we can avoid inducing overflow
-      ;; of the interrupt contexts. Relegating post-GC actions to their own thread
-      ;; (as suggested above) would largely solve this - the post-GC action would be to
-      ;; just "kick" that thread to do its thing. If it doesn't finish in time (by the
-      ;; time we kick it again), that's its problem, not the GC's problem.
-      ;;
-      ;; If we do implement a general post-GC thread, there remains a question of
-      ;; of what to do with post-GC actions for #-sb-thread that cons too much,
-      ;; risking recursive invocation of GC. Pretty much don't do that.
-      ;;
-      (when (and *allow-with-interrupts*
-                 ;; Continue if any of the following:
-                 ;; - you want a finalizer thread but it hasn't been started
-                 ;; - you don't want a finalizer thread, and we're not already
-                 ;;   in SCAN-FINALIZERS
-                 ;; - there are some hooks to run
-                 ;; Logically, the finalizer existence test belongs in src/code/final,
-                 ;; but the flaw in that is we're not going to call that code until unmasking
-                 ;; interrupts, which is precisely the thing we need to NOT do if already
-                 ;; in post-GC code of any kind (be it finalizer or other).
-                 (or #+sb-thread (if sb-impl::*finalizer-thread*
-                                     (not threadp)
-                                     (not sb-impl::*in-a-finalizer*))
-                     #-sb-thread (and (sb-impl::hash-table-culled-values
-                                       (sb-impl::finalizer-id-map
-                                        sb-impl::**finalizer-store**))
-                                      (not sb-impl::*in-a-finalizer*))
-                     *after-gc-hooks*))
-        (sb-thread::without-thread-waiting-for ()
-         (with-interrupts
-           (unless threadp
-             (run-pending-finalizers))
-           (call-hooks "after-GC" *after-gc-hooks* :on-error :warn)))))))
+        (run-pending-finalizers)))
+    ;; Here's one reason that MAX_INTERRUPTS is as high as it is.
+    ;; If the thread that performed GC runs post-GC hooks which cons enough to
+    ;; cause another GC while in the hooks, then as soon as interrupts are allowed
+    ;; again, a GC can be invoked "recursively" - while there is a maybe_gc() on
+    ;; the C call stack. It's not even true that _this_ thread had to cons a ton -
+    ;; any thread could have, causing this thread to hit the GC trigger which sets the
+    ;; P-A interrupted bit which causes the interrupt instruction at the end of a
+    ;; pseudo-atomic sequence to take a signal hit. So with interrupts eanbled,
+    ;; we get back into the GC, which calls post-GC, which might cons ...
+    ;; See the example at the bottom of src/code/final for a clear picture.
+    ;;
+    ;; To mitigate that problem, if you have no hooks, we can avoid inducing overflow
+    ;; of the interrupt contexts. Relegating post-GC actions to their own thread
+    ;; (as suggested above) would largely solve this - the post-GC action would be to
+    ;; just "kick" that thread to do its thing. If it doesn't finish in time (by the
+    ;; time we kick it again), that's its problem, not the GC's problem.
+    ;;
+    ;; If we do implement a general post-GC thread, there remains a question of
+    ;; of what to do with post-GC actions for #-sb-thread that cons too much,
+    ;; risking recursive invocation of GC. Pretty much don't do that.
+    ;;
+    (when (and *allow-with-interrupts*
+               ;; Continue if any of the following:
+               ;; - you want a finalizer thread but it hasn't been started
+               ;; - you don't want a finalizer thread, and we're not already
+               ;;   in SCAN-FINALIZERS
+               ;; - there are some hooks to run
+               ;; Logically, the finalizer existence test belongs in src/code/final,
+               ;; but the flaw in that is we're not going to call that code until unmasking
+               ;; interrupts, which is precisely the thing we need to NOT do if already
+               ;; in post-GC code of any kind (be it finalizer or other).
+               (or #+sb-thread (if sb-impl::*finalizer-thread*
+                                   (not threadp)
+                                   (not sb-impl::*in-a-finalizer*))
+                   #-sb-thread (and (sb-impl::hash-table-culled-values
+                                     (sb-impl::finalizer-id-map
+                                      sb-impl::**finalizer-store**))
+                                    (not sb-impl::*in-a-finalizer*))
+                   *after-gc-hooks*))
+      (sb-thread::without-thread-waiting-for ()
+        (with-interrupts
+          (unless threadp
+            (run-pending-finalizers))
+          (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))))))
 
 ;;; This is the user-advertised garbage collection function.
 (defun gc (&key (full nil) (gen 0) &allow-other-keys)
