@@ -10,6 +10,18 @@
 ;;;; files for more information.
 
 (in-package "SB-IMPL")
+
+
+;;;; Destructing-bind
+
+(sb-xc:defmacro destructuring-bind (lambda-list expression &body body
+                                                           &environment env)
+  (declare (ignore env)) ; could be policy-sensitive (but isn't)
+  "Bind the variables in LAMBDA-LIST to the corresponding values in the
+tree structure resulting from the evaluation of EXPRESSION."
+  `(binding* ,(sb-c::expand-ds-bind lambda-list expression t nil)
+     ,@body))
+
 
 ;;;; DEFUN
 
@@ -346,7 +358,7 @@ evaluated as a PROGN."
     (prog-expansion-from-let varlist body-decls 'let*)))
 
 (sb-xc:defmacro prog1 (result &body body)
-  (let ((n-result (gensym)))
+  (let ((n-result (sb-xc:gensym)))
     `(let ((,n-result ,result))
        (progn
          ,@body
@@ -386,7 +398,7 @@ evaluated as a PROGN."
            ;; Preserve non-toplevelness of the form!
            (let ((car (car forms))) (if nested car `(the t ,car))))
           (t
-           (let ((n-result (gensym)))
+           (let ((n-result (sb-xc:gensym)))
              `(let ((,n-result ,(first forms)))
                 (if ,n-result
                     ,n-result
@@ -440,7 +452,7 @@ evaluated as a PROGN."
   ;; form will take longer than can be described as adequate, as the
   ;; optional dispatch mechanism for the M-V-B gets increasingly
   ;; hairy.
-  (let ((val (and (sb-xc:constantp n env) (constant-form-value n env))))
+  (let ((val (and (constantp n env) (constant-form-value n env))))
     (if (and (integerp val) (<= 0 val (or #+(or x86-64 arm64 riscv) ;; better DEFAULT-UNKNOWN-VALUES
                                           1000
                                           10))) ; Arbitrary limit.
@@ -491,13 +503,13 @@ evaluated as a PROGN."
     (let* ((func (if (listp test-form) (car test-form)))
            (new-test
             (if (and (typep func '(and symbol (not null)))
-                     (not (sb-xc:macro-function func env))
-                     (not (sb-xc:special-operator-p func))
+                     (not (macro-function func env))
+                     (not (special-operator-p func))
                      (proper-list-p (cdr test-form)))
                 ;; TEST-FORM is a function call. We do not attempt this
                 ;; if TEST-FORM is a macro invocation or special form.
                 `(,func ,@(mapcar (lambda (place)
-                                    (if (sb-xc:constantp place env)
+                                    (if (constantp place env)
                                         place
                                         (with-unique-names (temp)
                                           (bindings `(,temp ,place))
@@ -574,7 +586,7 @@ invoked. In that case it will store into PLACE and start over."
            (setf ,place (check-type-error ',place ,place ',type
                                           ,@(and type-string
                                                  `(,type-string)))))
-        (let ((value (gensym)))
+        (let ((value (sb-xc:gensym)))
           `(do ((,value ,place ,place))
                ((typep ,value ',type))
              (setf ,place
@@ -636,7 +648,7 @@ invoked. In that case it will store into PLACE and start over."
     (sb-c::warn-if-compiler-macro-dependency-problem name)
     ;; FIXME: warn about incompatible lambda list with
     ;; respect to parent function?
-    (setf (sb-xc:compiler-macro-function name) definition)
+    (setf (compiler-macro-function name) definition)
     name))
 
 ;;;; CASE, TYPECASE, and friends
@@ -824,8 +836,8 @@ symbol-case giving up: case=((V U) (F))
       (flet ((try-one-byte ()
                (let ((best-answer nil)
                      ;; "best" means smallest
-                     (best-max-bin-count sb-xc:most-positive-fixnum) ; sentinel value
-                     (best-average sb-xc:most-positive-fixnum))
+                     (best-max-bin-count most-positive-fixnum) ; sentinel value
+                     (best-average most-positive-fixnum))
                  (loop named try
                        for nbits from smallest-nbits to largest-nbits
                        do (let ((bin-counts (make-array (ash 1 nbits) :initial-element 0)))
@@ -834,8 +846,8 @@ symbol-case giving up: case=((V U) (F))
                        finally (return-from try (values best-max-bin-count best-answer)))))
              (try-two-bytes ()
                (let ((best-answer nil)
-                     (best-max-bin-count sb-xc:most-positive-fixnum)
-                     (best-average sb-xc:most-positive-fixnum))
+                     (best-max-bin-count most-positive-fixnum)
+                     (best-average most-positive-fixnum))
                  (loop named try
                        for nbits from smallest-nbits to largest-nbits
                        do (let ((bin-counts (make-array (ash 1 nbits) :initial-element 0)))
@@ -1467,9 +1479,9 @@ symbol-case giving up: case=((V U) (F))
                   (setq clauses (if default (cons default new-clauses) new-clauses)
                         keys (new-keys)
                         implement-as 'case))
-                 #+(vop-named sb-c:multiway-branch-if-eq)
-                 ((expand-struct-typecase keyform keyform-value normal-clauses keys
-                                          default errorp)
+                 ((and (sb-c::vop-existsp :named sb-c:multiway-branch-if-eq)
+                       (expand-struct-typecase keyform keyform-value normal-clauses keys
+                                          default errorp))
                   (return-from case-body-aux it))))))
 
     ;; Efficiently expanding CASE over symbols depends on CASE over integers being
@@ -1555,7 +1567,7 @@ symbol-case giving up: case=((V U) (F))
 (sb-xc:defmacro with-open-file ((stream filespec &rest options)
                                 &body body)
   (multiple-value-bind (forms decls) (parse-body body nil)
-    (let ((abortp (gensym)))
+    (let ((abortp (sb-xc:gensym)))
       `(let ((,stream (open ,filespec ,@options))
              (,abortp t))
          ,@decls
@@ -1565,41 +1577,6 @@ symbol-case giving up: case=((V U) (F))
                 (setq ,abortp nil))
            (when ,stream
              (close ,stream :abort ,abortp)))))))
-
-(sb-xc:defmacro with-output-to-string
-    ((var &optional string &key (element-type ''character))
-     &body forms-decls)
-  (multiple-value-bind (forms decls) (parse-body forms-decls nil)
-    (if string
-        (let ((element-type-var (gensym)))
-          `(let ((,var (make-fill-pointer-output-stream ,string))
-                 ;; ELEMENT-TYPE isn't currently used for anything
-                 ;; (see FILL-POINTER-OUTPUT-STREAM FIXME in stream.lisp),
-                 ;; but it still has to be evaluated for side-effects.
-                 (,element-type-var ,element-type))
-             (declare (ignore ,element-type-var))
-             ,@decls
-             (unwind-protect
-                  (progn ,@forms)
-               (close ,var))))
-        ;; I don't see why we need the unwind-protect.
-        ;; CLHS says: "The output string stream to which the variable /var/
-        ;; is bound has dynamic extent; its extent ends when the form is exited."
-        ;; So technically you can't reference the stream after the string
-        ;; is returned. And since the implication is that we can legally DXify
-        ;; the stream, what difference does it make whether it's open or closed?
-        ;; Certainly in code that is compiled in SAFETY < 3 we should
-        ;; just not bother with the unwind-protect or the close.
-        `(let ((,var (make-string-output-stream
-                      ;; CHARACTER is the default element-type of
-                      ;; string-ouput-stream, save a few bytes when passing it
-                      ,@(and (not (equal element-type ''character))
-                             `(:element-type ,element-type)))))
-           ,@decls
-           (unwind-protect
-                (progn ,@forms)
-             (close ,var))
-           (get-output-stream-string ,var)))))
 
 ;;;; Iteration macros:
 
@@ -1693,12 +1670,12 @@ symbol-case giving up: case=((V U) (F))
   ;; since we don't want to use IGNORABLE on what might be a special
   ;; var.
   (binding* (((forms decls) (parse-body body nil))
-             (n-list (gensym "N-LIST"))
-             (start (gensym "START"))
+             (n-list (sb-xc:gensym "N-LIST"))
+             (start (sb-xc:gensym "START"))
              ((clist members clist-ok)
               (with-current-source-form (list)
                 (cond
-                  ((sb-xc:constantp list env)
+                  ((constantp list env)
                    (binding* ((value (constant-form-value list env))
                               ((all dot) (list-members value :max-length 20)))
                      (when (eql dot t)
@@ -1709,7 +1686,7 @@ symbol-case giving up: case=((V U) (F))
                          (values value nil nil)
                          (values value all t))))
                   ((and (consp list) (eq 'list (car list))
-                        (every (lambda (arg) (sb-xc:constantp arg env)) (cdr list)))
+                        (every (lambda (arg) (constantp arg env)) (cdr list)))
                    (let ((values (mapcar (lambda (arg) (constant-form-value arg env)) (cdr list))))
                      (values values values t)))
                   (t
@@ -1756,9 +1733,6 @@ symbol-case giving up: case=((V U) (F))
                  `(sb-c::%proclaim ',spec (sb-c:source-location)))
                specs)))
 
-;; Avoid unknown return values in emitted code for PRINT-UNREADABLE-OBJECT
-(sb-xc:proclaim '(ftype (sfunction (t t t &optional t) null)
-                        %print-unreadable-object))
 (sb-xc:defmacro print-unreadable-object ((object stream &key type identity)
                                              &body body)
   "Output OBJECT to STREAM with \"#<\" prefix, \">\" suffix, optionally
@@ -1809,3 +1783,34 @@ symbol-case giving up: case=((V U) (F))
                     ;; is cheaper than a branch around a write.
                     (rplacd (truly-the cons ,splice) ,orig)))
             ,copy))))))
+
+(defun expand-with-output-to-string (var element-type body wild-result-type)
+  ;; This is simpler than trying to arrange transforms that cause
+  ;; MAKE-STRING-OUTPUT-STREAM to be DXable. While that might be awesome,
+  ;; this macro exists for a reason.
+  (let ((initial-buffer '#:buf)
+        (dummy '#:stream)
+        (string-let (or #+c-stack-is-control-stack 'dx-let 'let))
+        (string-ctor
+          (if (and (sb-xc:constantp element-type)
+                   (let ((ctype (sb-c::careful-specifier-type
+                                 (constant-form-value element-type))))
+                     (and ctype
+                          (csubtypep ctype (specifier-type 'character))
+                          (neq ctype *empty-type*))))
+              ;; Using MAKE-ARRAY avoids a style-warning if et is 'STANDARD-CHAR:
+              ;; "The default initial element #\Nul is not a STANDARD-CHAR."
+              'make-array ; hooray! it's known be a valid string type
+              ;; Force a runtime STRINGP check unless futher transforms
+              ;; deduce a known type. You'll get "could not stack allocate"
+              ;; perhaps, but that's acceptable.
+              'make-string)))
+    ;; A full call to MAKE-STRING-OUTPUT-STREAM uses a larger initial buffer
+    ;; if BASE-CHAR but I really don't care to think about that here.
+    `(,string-let ((,initial-buffer (,string-ctor 31 :element-type ,element-type)))
+       (dx-let ((,dummy (%allocate-string-ostream)))
+         (let ((,var (%init-string-output-stream ,dummy ,initial-buffer
+                                                 ,wild-result-type)))
+           (declare (ignorable ,var))
+           ,@body)
+         (get-output-stream-string ,dummy)))))
