@@ -165,7 +165,7 @@
 ;;; mutex is needed. More importantly the sigchld signal handler also
 ;;; accesses it, that's why we need without-interrupts.
 (defmacro with-active-processes-lock (() &body body)
-  `(sb-thread::with-system-mutex (*active-processes-lock*)
+  `(with-system-mutex (*active-processes-lock*)
      ,@body))
 
 (deftype process-status ()
@@ -447,14 +447,14 @@ status slot."
     ;; No dice, try using the old-school method.
     (dolist (char '(#\p #\q))
       (dotimes (digit 16)
-        (let* ((master-name (with-simple-output-to-string (str nil base-char)
+        (let* ((master-name (with-output-to-string (str nil :element-type 'base-char)
                               (format str "/dev/pty~C~X" char digit)))
                (master-fd (sb-unix:unix-open master-name
                                              (logior sb-unix:o_rdwr
                                                      sb-unix:o_noctty)
                                              #o666)))
           (when master-fd
-            (let* ((slave-name (with-simple-output-to-string (str nil base-char)
+            (let* ((slave-name (with-output-to-string (str nil :element-type 'base-char)
                                  (format str "/dev/tty~C~X" char digit)))
                    (slave-fd (sb-unix:unix-open slave-name
                                                 (logior sb-unix:o_rdwr
@@ -630,7 +630,7 @@ status slot."
 
 #+win32
 (defun prepare-args (args escape)
-  (with-simple-output-to-string (str)
+  (%with-output-to-string (str)
     (loop for (arg . rest) on args
           do
           (cond ((and escape
@@ -711,7 +711,8 @@ status slot."
                     status-hook
                     (external-format :default)
                     directory
-                    #+win32 (escape-arguments t))
+                    #+win32 (escape-arguments t)
+                    #+win32 (window nil))
   "RUN-PROGRAM creates a new process specified by PROGRAM.
 ARGS are passed as the arguments to the program.
 
@@ -801,7 +802,16 @@ Users Manual for details about the PROCESS structure.
 
    Windows specific options:
    :ESCAPE-ARGUMENTS (default T)
-      Controls escaping of the arguments passed to CreateProcess."
+      Controls escaping of the arguments passed to CreateProcess.
+   :WINDOW (default NIL)
+      When NIL, the subprocess decides how it will display its window. The
+      following options control how the subprocess window should be displayed:
+      :HIDE, :SHOW-NORMAL, :SHOW-MAXIMIZED, :SHOW-MINIMIZED, :SHOW-NO-ACTIVATE,
+      :SHOW-MIN-NO-ACTIVE, :SHOW-NA.
+      Note: console application subprocesses may or may not display a console
+      window depending on whether the SBCL runtime is itself a console or GUI
+      application. Invoke CMD /C START to consistently display a console window
+      or use the :WINDOW :HIDE option to consistently hide the console window."
   (when (and env-p environment-p)
     (error "can't specify :ENV and :ENVIRONMENT simultaneously"))
   (let* (;; Clear various specials used by GET-DESCRIPTOR-FOR to
@@ -871,15 +881,16 @@ Users Manual for details about the PROCESS structure.
                        (with-open-pty ((pty-name pty-stream) (pty cookie))
                          (setf (values child #+win32 handle)
                                #+win32
-                               (sb-thread::with-system-mutex (*spawn-lock*)
+                               (with-system-mutex (*spawn-lock*)
                                  (sb-win32::mswin-spawn
                                   progname
                                   args
                                   stdin stdout stderr
-                                  search environment-vec directory))
+                                  search environment-vec directory
+                                  window))
                                #-win32
                                (with-args (args-vec args)
-                                 (sb-thread::with-system-mutex (*spawn-lock*)
+                                 (with-system-mutex (*spawn-lock*)
                                    (spawn progname args-vec
                                           stdin stdout stderr
                                           (if search 1 0)
@@ -1279,14 +1290,14 @@ Users Manual for details about the PROCESS structure.
           (t
            (fail "invalid option: ~S" object))))))
 
-#+(or linux sunos hpux haiku)
+#+(or linux sunos haiku)
 (defun software-version ()
   "Return a string describing version of the supporting software, or NIL
   if not available."
   (or sb-sys::*software-version*
       (setf sb-sys::*software-version*
             (string-trim '(#\newline)
-                         (with-simple-output-to-string (stream)
+                         (%with-output-to-string (stream)
                            (run-program "/bin/uname"
                                         ;; "-r" on haiku just prints "1"
                                         ;; but "-v" prints some detail.
