@@ -9,6 +9,8 @@
 
 (in-package "SB-KERNEL")
 
+#-sb-devel(declaim (start-block))
+
 ;;; (Note that when cross-compiling, SB-XC:TYPEP is interpreted as a
 ;;; test that the host Lisp object OBJECT translates to a target SBCL
 ;;; type TYPE. This behavior is needed e.g. to test for the validity
@@ -156,7 +158,7 @@
                          (typecase object
                            (instance (%instance-layout object))
                            (funcallable-instance
-                            (%funcallable-instance-layout object))
+                            (%fun-layout object))
                            (t (return)))
                          (cdr (truly-the cons cache))
                          object)))
@@ -172,7 +174,7 @@
 (defun classoid-cell-typep (cell object)
   (let ((layout (typecase object
                   (instance (%instance-layout object))
-                  (funcallable-instance (%funcallable-instance-layout object))
+                  (funcallable-instance (%fun-layout object))
                   (t (return-from classoid-cell-typep))))
         (classoid (classoid-cell-classoid cell)))
     (unless classoid
@@ -180,6 +182,23 @@
              (classoid-cell-name cell)))
     (classoid-typep layout classoid object)))
 
+;;; Return true of any object which is either a funcallable-instance,
+;;; or an ordinary instance that is not a structure-object.
+(declaim (inline %pcl-instance-p))
+(defun %pcl-instance-p (x)
+  ;; The COND is more efficient than LAYOUT-OF.
+  (layout-for-pcl-obj-p (cond ((%instancep x) (%instance-layout x))
+                              ((function-with-layout-p x) (%fun-layout x))
+                              (t (return-from %pcl-instance-p nil)))))
+
+;;; Try to ensure that the object's layout is up-to-date only if it is an instance
+;;; or funcallable-instance of other than a static or structure classoid type.
+;;; LAYOUT is the *expected* type, not the the layout currently in OBJECT.
+(defun update-object-layout-or-invalid (object layout)
+  (if (%pcl-instance-p object)
+      (sb-pcl::check-wrapper-validity object)
+      (sb-c::%layout-invalid-error object layout)))
+
 ;;; Test whether OBJ-LAYOUT is from an instance of CLASSOID.
 
 ;;; IMPORTANT: If none of the classes involved (directly or indirectly)
@@ -227,7 +246,7 @@
            (values obj-layout layout))
         (aver (< i 2))
         (%ensure-classoid-valid classoid layout "typep")
-        (when (layout-invalid obj-layout)
+        (when (zerop (layout-clos-hash obj-layout))
           (setq obj-layout (update-object-layout-or-invalid object layout))))
     (or (eq obj-layout layout)
         (let ((obj-inherits (layout-inherits obj-layout)))
@@ -235,6 +254,8 @@
             (when (eq (svref obj-inherits i) layout)
               (return t)))))))
 
+(declaim (end-block))
+
 ;;; If TYPE is a type that we can do a compile-time test on, then
 ;;; return whether the object is of that type as the first value and
 ;;; second value true. Otherwise return NIL, NIL.
@@ -279,8 +300,22 @@
     (alien-type-type
      (values (alien-typep obj (alien-type-type-alien-type type)) t))
     (hairy-type
+     ;; Parse it again to make sure it's really undefined.
+     ;; FIXME: This logic also appears in %%TYPEP, and probably needs
+     ;; to happen in more places too. (Like the array hairy type
+     ;; testing.)
      (if (unknown-type-p type)
-         (values nil nil)
+         (let ((spec (unknown-type-specifier type)))
+           ;; KLUDGE: Work around the fact that we load PCL after we
+           ;; type test certain things defined there. This is somewhat
+           ;; suboptimal because this is really only a concern during
+           ;; warm load.
+           (if (member spec '(class sb-pcl::condition-class))
+               (values nil nil)
+               (let ((type (specifier-type spec)))
+                 (if (unknown-type-p type)
+                     (values nil nil)
+                     (ctypep obj type)))))
          ;; Now the tricky stuff.
          (let ((predicate (cadr (hairy-type-specifier type))))
            (case predicate
