@@ -381,10 +381,18 @@
 
 ;;; Just convert it into a MAKE-ARRAY.
 (deftransform make-string ((length &key
-                                   (element-type 'character)
+                                   element-type
                                    (initial-element (code-char 0))))
+  ;; There's a minor edge case that is debatable: if you specify a type that is not
+  ;; spelled 'NIL, but equates to the empty type, then we should still return a
+  ;; string. MAKE-ARRAY won't do that. The safe thing to do would be to abort this
+  ;; transform if we don't see a constant type that is known not to be empty.
+  ;; But it's unnecessary hair-splitting: the old behavior was to produce a string
+  ;; that for all intents and purposes was unusable; the new behavior is to similarly
+  ;; produce an unusable object, but signal a TYPE-ERROR.
+  ;; The lisp hackers I surveyed would rather see the error sooner than later.
   `(the simple-string (make-array (the index length)
-                       :element-type element-type
+                       :element-type (or element-type 'character)
                        ,@(when initial-element
                            '(:initial-element initial-element)))))
 
@@ -589,6 +597,7 @@
                                 (t '*))
                          ,(if dims-constp dims (length dims)))
               (make-array-header*
+               sb-vm:simple-array-widetag
                ,@(sb-vm::make-array-header-inits
                   `(make-array ,size ,@keys) size dims)))))
       `(let* (,@axis-bindings ,@et-binding (,size (the index (* ,@dims))))
@@ -645,22 +654,16 @@
   (let* ((c-length (if (lvar-p length)
                        (if (constant-lvar-p length) (lvar-value length))
                        length))
-         (complex (cond ((and (or
-                               (not fill-pointer)
-                               (and (constant-lvar-p fill-pointer)
-                                    (null (lvar-value fill-pointer))))
-                              (or
-                               (not adjustable)
-                               (and
-                                (constant-lvar-p adjustable)
-                                (null (lvar-value adjustable)))))
-                         nil)
-                        ((and (constant-lvar-p adjustable)
-                              (lvar-value adjustable)))
-                        ((and fill-pointer
-                              (constant-lvar-p fill-pointer)
-                              (lvar-value fill-pointer)))
-                        (t
+         (expressly-adjustable (cond ((not adjustable) nil)
+                                     ((not (constant-lvar-p adjustable)) :maybe)
+                                     (t (and (lvar-value adjustable) t))))
+         (has-fill-pointer (cond ((not fill-pointer) nil)
+                                 ((numeric-type-p (lvar-type fill-pointer)) t)
+                                 ((constant-lvar-p fill-pointer)
+                                  (not (null (lvar-value fill-pointer))))
+                                 (t :maybe)))
+         (complex (cond ((or (eq expressly-adjustable t) (eq has-fill-pointer t)) t)
+                        ((or (eq expressly-adjustable :maybe) (eq has-fill-pointer :maybe))
                          ;; Deciding between complex and simple at
                          ;; run-time would be too much hassle
                          (give-up-ir1-transform))))
@@ -724,7 +727,8 @@
       (abort-ir1-transform "Invalid fill-pointer ~s for a vector of length ~s."
                            (type-specifier (lvar-type fill-pointer))
                            c-length))
-    (flet ((eliminate-keywords ()
+    (labels
+          ((eliminate-keywords ()
              (eliminate-keyword-args
               call 1
               '((:element-type element-type)
@@ -732,47 +736,46 @@
                 (:initial-element initial-element)
                 (:adjustable adjustable)
                 (:fill-pointer fill-pointer))))
+           (allocator (type-spec &rest rest)
+             (let ((array
+                    `(make-array-header*
+                      ,(logior (if (eq has-fill-pointer t) ; can't handle :maybe
+                                   (ash sb-vm:+array-fill-pointer-p+ sb-vm:n-widetag-bits)
+                                   0)
+                               (or (sb-vm:saetp-complex-typecode saetp)
+                                   sb-vm:complex-vector-widetag))
+                       ,@rest)))
+               (if (eq has-fill-pointer :maybe)
+                   `(let ((%array (truly-the ,type-spec ,array)))
+                      (when fill-pointer
+                        (set-header-bits %array sb-vm:+array-fill-pointer-p+))
+                      %array)
+                   `(truly-the ,type-spec ,array))))
            (with-alloc-form (&optional data-wrapper)
              (cond (complex
                     (let* ((constant-fill-pointer-p (constant-lvar-p fill-pointer))
                            (fill-pointer-value (and constant-fill-pointer-p
-                                                    (lvar-value fill-pointer))))
+                                                    (lvar-value fill-pointer)))
+                           (length-expr
+                            (cond ((eq fill-pointer-value t) '%length)
+                                  (fill-pointer-value)
+                                  ((and fill-pointer (not constant-fill-pointer-p))
+                                   `(cond ((or (eq fill-pointer t) (null fill-pointer))
+                                           %length)
+                                          ((> fill-pointer %length)
+                                           (error "Invalid fill-pointer ~a" fill-pointer))
+                                          (t fill-pointer)))
+                                  (t '%length))))
                       `(let ((%length (the index ,(or c-length 'length))))
-                         (truly-the
-                          ,result-spec
-                          (make-array-header* ,(or (sb-vm:saetp-complex-typecode saetp)
-                                                   sb-vm:complex-vector-widetag)
-                                              ;; fill-pointer
-                                              ,(cond ((eq fill-pointer-value t)
-                                                      '%length)
-                                                     (fill-pointer-value)
-                                                     ((and fill-pointer
-                                                           (not constant-fill-pointer-p))
-                                                      `(cond ((or (eq fill-pointer t)
-                                                                  (null fill-pointer))
-                                                              %length)
-                                                             ((> fill-pointer %length)
-                                                              (error "Invalid fill-pointer ~a" fill-pointer))
-                                                             (t
-                                                              fill-pointer)))
-                                                     (t
-                                                      '%length))
-                                              ;; fill-pointer-p
-                                              ,(and fill-pointer
-                                                    `(and fill-pointer t))
-                                              ;; elements
-                                              %length
-                                              ;; data
-                                              (let ((data ,data-alloc-form))
-                                                ,(or data-wrapper 'data))
-                                              ;; displacement
-                                              0
-                                              ;; displaced-p
-                                              nil
-                                              ;; displaced-from
-                                              nil
-                                              ;; dimensions
-                                              %length)))))
+                         ,(allocator result-spec
+                                     length-expr   ; fill-pointer
+                                     '%length      ; elements
+                                     `(let ((data ,data-alloc-form)) ; data
+                                        ,(or data-wrapper 'data))
+                                     0             ; displacement
+                                     nil           ; displaced-p
+                                     nil           ; displaced-from
+                                     '%length))))  ; dimensions
                    (data-wrapper
                     (subst data-alloc-form 'data data-wrapper))
                    (t
@@ -861,7 +864,16 @@
              #-sb-xc-host
              (and (and (testable-type-p elt-ctype)
                        (neq elt-ctype *empty-type*)
-                       (not (ctypep default-initial-element elt-ctype)))
+                       ;; An explicitly passed :INITIAL-ELEMENT must match the specified
+                       ;; element-type, not the upgraded type. But I don't want to style-warn
+                       ;; about (make-array 1 :element-type 'standard-char) with an implicit
+                       ;; initial element of #\nul. However, generally comparing against the
+                       ;; upgraded type for any implicit initializer would never warn about
+                       ;; things like (make-array 1 :element-type 'string).
+                       (not (ctypep default-initial-element
+                                    (if (and (eq elt-spec 'standard-char) (not initial-element))
+                                        (sb-vm:saetp-ctype saetp)
+                                        elt-ctype))))
                   ;; This situation arises e.g. in (MAKE-ARRAY 4 :ELEMENT-TYPE
                   ;; '(INTEGER 1 5)) ANSI's definition of MAKE-ARRAY says "If
                   ;; INITIAL-ELEMENT is not supplied, the consequences of later
@@ -1087,8 +1099,6 @@
                                                     sb-vm:simple-array-widetag)
                                                ;; fill-pointer
                                                ,total-size
-                                               ;; fill-pointer-p
-                                               nil
                                                ;; elements
                                                ,total-size
                                                ;; data
@@ -1160,7 +1170,7 @@
 
 ;;;; miscellaneous properties of arrays
 
-;;; Transforms for various array properties. If the property is know
+;;; Transforms for various array properties. If the property is known
 ;;; at compile time because of a type spec, use that constant value.
 
 ;;; Most of this logic may end up belonging in code/late-type.lisp;
@@ -1235,14 +1245,22 @@
 ;; reduction.
 (deftransform array-rank ((array) (array) * :node node)
   (let ((array-type (lvar-type array)))
-    (cond ((eq t (and (array-type-p array-type)
-                      (array-type-complexp array-type)))
+    (cond ((and (array-type-p array-type)
+                (listp (array-type-dimensions array-type)))
+           (length (array-type-dimensions array-type)))
+          ;; We need an extra case in here to best handle a known vector, because
+          ;; if we try to add a transform on ARRAY-HEADER-P (returning false for
+          ;; known simple-array of rank 1), that would fail to optimize where we know
+          ;; the thing is a vector but possibly non-simple. So then the IF below
+          ;; would still have both if its consequents considered plausible.
+          ((csubtypep array-type (specifier-type 'vector))
+           1)
+          ((and (array-type-p array-type)
+                (eq (array-type-complexp array-type) t))
            '(%array-rank array))
           (t
-           (delay-ir1-transform node :constraint)
-           `(if (array-header-p array)
-                (%array-rank array)
-                1)))))
+           (delay-ir1-transform node :constraint) ; Why?
+           `(%array-rank array)))))
 
 (defun derive-array-rank (ctype)
   (let ((array (specifier-type 'array)))
@@ -1404,21 +1422,24 @@
           (t
            `(%array-available-elements array)))))
 
-;;; Only complex vectors have fill pointers.
+;;; Any array can be tested for a fill-pointer now, using the header bit.
+;;; Only a non-simple vector could possibly return true.
+;;; If the input is known simple, we have to avoid doing the logtest because there's
+;;; no constraint that says that the logtest will return false, and style-warnings
+;;; will result from MAP-INTO and other things which have
+;;    (if (array-has-fill-pointer-p a) (setf (fill-pointer a) ...))
+;; where the compiler knows that the input is simple.
 (deftransform array-has-fill-pointer-p ((array))
   (let* ((array-type (lvar-type array))
          (dims (array-type-dimensions-or-give-up array-type)))
-    (if (and (listp dims) (not (= (length dims) 1)))
-        nil
-        (ecase (conservative-array-type-complexp array-type)
-          ((t)
-           t)
-          ((nil)
-           nil)
-          ((:maybe)
-           (give-up-ir1-transform
-            "The array type is ambiguous; must call ~
-               ARRAY-HAS-FILL-POINTER-P at runtime."))))))
+    ;; If a vector and possibly non-simple, then perform the bit test,
+    ;; otherwise the answer is definitely NIL.
+    (cond ((and (listp dims) (/= (length dims) 1)) nil) ; dims = * is possibly a vector
+          ((eq (conservative-array-type-complexp array-type) nil) nil)
+          (t
+           (if (vop-existsp :named test-header-bit)
+               `(test-header-bit array sb-vm:+array-fill-pointer-p+)
+               `(logtest (get-header-data array) sb-vm:+array-fill-pointer-p+))))))
 
 (deftransform %check-bound ((array dimension index) ((simple-array * (*)) * *))
   (let ((array-ref (lvar-uses array))
@@ -1888,7 +1909,9 @@
   (declare (ignore gen))
   (values array (specifier-type '(and array (not (simple-array * (*)))))))
 
-(defoptimizer (%array-fill-pointer-p constraint-propagate-if)
+;;; If ARRAY-HAS-FILL-POINTER-P returns true, then ARRAY
+;;; is of the specified type.
+(defoptimizer (array-has-fill-pointer-p constraint-propagate-if)
     ((array) node gen)
   (declare (ignore gen))
   (values array (specifier-type '(and vector (not simple-array)))))
