@@ -1359,7 +1359,7 @@
     (incf n)
     (binding* ((vop (stmt-vop statement) :exit-if-null))
       (unless (eq vop last-vop)
-        (format stream "## ~A~%" (sb-c::vop-info-name (sb-c::vop-info vop))))
+        (format stream "## ~A~%" (sb-c::vop-name vop)))
       (setq last-vop vop))
     (let ((op (stmt-mnemonic statement))
           (eol-comment ""))
@@ -1402,7 +1402,7 @@
 
 ;;; Map of opcode symbol to function that emits it into the byte stream
 ;;; (or with the schedulding assembler, into the queue)
-(defglobal *inst-encoder* (make-hash-table :test 'eq))
+(defglobal *inst-encoder* (make-hash-table)) ; keys are symbols
 
 ;;; Combine INPUTS into one assembly stream and assemble into SEGMENT
 (defun %assemble (segment section)
