@@ -12,16 +12,30 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; utilities used during code generation
 
 ;;; the number of bytes used by the code object header
+
+#-sb-xc-host
 (defun component-header-length (&optional
                                 (component *component-being-compiled*))
   (let* ((2comp (component-info component))
          (constants (ir2-component-constants 2comp)))
-    (ash (align-up (length constants) code-boxed-words-align) sb!vm:word-shift)))
+    (ash (align-up (length constants) code-boxed-words-align) sb-vm:word-shift)))
+
+;;; KLUDGE: the assembler can not emit backpatches comprising jump tables without
+;;; knowing the boxed code header length. But there is no compiler IR2 metaobject,
+;;; for SB-FASL:*ASSEMBLER-ROUTINES*. We have to return a fixed answer for that.
+#+sb-xc-host
+(defun component-header-length ()
+  (if (boundp '*component-being-compiled*)
+      (let ((component *component-being-compiled*))
+        (let* ((2comp (component-info component))
+               (constants (ir2-component-constants 2comp)))
+          (ash (align-up (length constants) code-boxed-words-align) sb-vm:word-shift)))
+      (* sb-vm:n-word-bytes 4)))
 
 ;;; the size of the NAME'd SB in the currently compiled component.
 ;;; This is useful mainly for finding the size for allocating stack
@@ -86,23 +100,91 @@
            (block-home-lambda
             (block-next (component-head *component-being-compiled*))))
           (or (> speed compilation-speed) (> space compilation-speed))))
-(defun default-segment-inst-hook ()
-  (and *compiler-trace-output*
-       #'trace-instruction))
 
+;;; Some platforms support unboxed constants immediately following the boxed
+;;; code header. Such platform must implement supporting 4 functions:
+;;; * CANONICALIZE-INLINE-CONSTANT: converts a constant descriptor (list) into
+;;;    a canonical description, to be used as a key in an EQUAL hash table
+;;;    and to guide the generation of the constant itself.
+;;; * INLINE-CONSTANT-VALUE: given a canonical constant descriptor, computes
+;;;    two values:
+;;;     1. A label that will be used to emit the constant (usually a
+;;;         sb-assem:label)
+;;;     2. A value that will be returned to code generators referring to
+;;;         the constant (on x86oids, an EA object)
+;;; * SORT-INLINE-CONSTANTS: Receives a vector of unique constants;
+;;;    the car of each entry is the constant descriptor, and the cdr the
+;;;    corresponding label. Destructively returns a vector of constants
+;;;    sorted in emission order. It could actually perform arbitrary
+;;;    modifications to the vector, e.g. to fuse constants of different
+;;;    size.
+;;; * EMIT-INLINE-CONSTANT: receives a constant descriptor and its associated
+;;;    label. Emits the constant.
+;;;
+;;; Implementing this feature lets VOP generators use sb-c:register-inline-constant
+;;; to get handles (as returned by sb-vm:inline-constant-value) from constant
+;;; descriptors.
+;;;
+#+(or x86 x86-64 arm64 ppc ppc64)
+(defun register-inline-constant (&rest constant-descriptor)
+  (declare (dynamic-extent constant-descriptor))
+  (let ((asmstream *asmstream*)
+        (constant (sb-vm:canonicalize-inline-constant constant-descriptor)))
+    (ensure-gethash
+     constant
+     (asmstream-constant-table asmstream)
+     (multiple-value-bind (label value) (sb-vm:inline-constant-value constant)
+       (vector-push-extend (cons constant label)
+                           (asmstream-constant-vector asmstream))
+       value))))
+#-(or x86 x86-64 arm64 ppc ppc64)
+(progn (defun sb-vm:sort-inline-constants (constants) constants)
+       (defun sb-vm:emit-inline-constant (&rest args)
+         (error "EMIT-INLINE-CONSTANT called with ~S" args)))
+;;; Emit the subset of inline constants which represent jump tables
+;;; and remove those constants so that they don't get emitted again.
+(defun emit-jump-tables ()
+  ;; Other backends will probably need relative jump tables instead
+  ;; of absolute tables because of the problem of needing to load
+  ;; the LIP register prior to loading an arbitrary PC.
+  (let* ((asmstream *asmstream*)
+         (constants (asmstream-constant-vector asmstream))
+         (section (asmstream-data-section asmstream))
+         (nwords 0))
+    (collect ((jump-tables) (other))
+      (dovector (constant constants)
+        ;; Constant is ((category . data) . label)
+        (cond ((eq (caar constant) :jump-table)
+               (incf nwords (length (cdar constant)))
+               (jump-tables constant))
+              (t
+               (other constant))))
+      ;; Preface the unboxed data with a count of jump-table words,
+      ;; including the count itself as 1 word.
+      ;; On average this will add another padding word half of the time
+      ;; depending on the number of boxed constants. We could reduce space
+      ;; by storing one bit in the header indicating whether or not there
+      ;; is a jump table. I don't think that's worth the trouble.
+      (emit section `(.lispword ,(1+ nwords)))
+      (when (plusp nwords)
+        (dolist (constant (jump-tables))
+          (sb-vm:emit-inline-constant section (car constant) (cdr constant)))
+        (let ((nremaining (length (other))))
+          (adjust-array constants nremaining
+                        :fill-pointer nremaining
+                        :initial-contents (other)))))))
 ;;; Return T if and only if there were any constants emitted.
 (defun emit-inline-constants ()
-  #!+inline-constants
   (let* ((asmstream *asmstream*)
-         (sorted (sb!vm:sort-inline-constants
-                  (asmstream-constant-vector asmstream)))
+         (constants (asmstream-constant-vector asmstream))
          (section (asmstream-data-section asmstream)))
-    (dovector (constant sorted (plusp (length sorted)))
-      (sb!vm:emit-inline-constant section (car constant) (cdr constant)))))
+    (when (plusp (length constants))
+      (dovector (constant (sb-vm:sort-inline-constants constants) t)
+        (sb-vm:emit-inline-constant section (car constant) (cdr constant))))))
 
 ;;; If a constant is already loaded into a register use that register.
 (defun optimize-constant-loads (component)
-  (let* ((register-sb (sb-or-lose 'sb!vm::registers))
+  (let* ((register-sb (sb-or-lose 'sb-vm::registers))
          (loaded-constants
            (make-array (sb-size register-sb)
                        :initial-element nil)))
@@ -139,10 +221,10 @@
                             (remove-constant (tn-ref-load-tn ref))))))
                  (compatible-scs-p (a b)
                    (or (eql a b)
-                       (and (eq (sc-name a) 'sb!vm::control-stack)
-                            (eq (sc-name b) 'sb!vm::descriptor-reg))
-                       (and (eq (sc-name b) 'sb!vm::control-stack)
-                            (eq (sc-name a) 'sb!vm::descriptor-reg))))
+                       (and (eq (sc-name a) 'sb-vm::control-stack)
+                            (eq (sc-name b) 'sb-vm::descriptor-reg))
+                       (and (eq (sc-name b) 'sb-vm::control-stack)
+                            (eq (sc-name a) 'sb-vm::descriptor-reg))))
                  (find-constant-tn (constant sc)
                    (loop for (saved-constant . tn) across loaded-constants
                          when (and saved-constant
@@ -150,7 +232,7 @@
                                    (compatible-scs-p (tn-sc tn) sc))
                          return tn)))
           (case (vop-name vop)
-            ((move sb!vm::move-arg)
+            ((move sb-vm::move-arg)
              (let* ((args (vop-args vop))
                     (x (tn-ref-tn args))
                     (y (tn-ref-tn (vop-results vop)))
@@ -180,6 +262,7 @@
 ;; Collect "static" count of number of times each vop is employed.
 ;; (as opposed to "dynamic" - how many times its code is hit at runtime)
 (defglobal *static-vop-usage-counts* nil)
+(defparameter *do-instcombine-pass* t)
 
 (defun generate-code (component)
   (when *compiler-trace-output*
@@ -203,6 +286,8 @@
             ;; on 8 byte boundaries we cannot guarantee proper loop alignment
             ;; there (yet.)  Only x86-64 does something with ALIGNP, but
             ;; it may be useful in the future.
+            ;; FIXME: see comment in ASSEMBLE-SECTIONS - we *can* enforce larger
+            ;; alignment than the size of a cons cell.
             (let ((alignp (let ((cloop (block-loop 1block)))
                             (when (and cloop
                                        (loop-tail cloop)
@@ -216,7 +301,7 @@
                                  alignp)))
           (let ((env (block-physenv 1block)))
             (unless (eq env prev-env)
-              (let ((lab (gen-label)))
+              (let ((lab (gen-label "physenv elsewhere start")))
                 (setf (ir2-physenv-elsewhere-start (physenv-info env))
                       lab)
                 (emit (asmstream-elsewhere-section asmstream) lab))
@@ -225,98 +310,51 @@
           ((null vop))
         (let ((gen (vop-info-generator-function (vop-info vop))))
           (awhen *static-vop-usage-counts*
-            (let ((name (vop-info-name (vop-info vop))))
+            (let ((name (vop-name vop)))
               (incf (gethash name it 0))))
           (assemble (:code vop)
             (cond ((not gen)
                    (format t
                            "missing generator for ~S~%"
                            (template-name (vop-info vop))))
-                  #!+arm64
+                  #+arm64
                   ((and (vop-next vop)
                         (eq (vop-name vop)
                             (vop-name (vop-next vop)))
-                        (memq (vop-name vop) '(move move-operand sb!vm::move-arg))
-                        (sb!vm::load-store-two-words vop (vop-next vop)))
+                        (memq (vop-name vop) '(move move-operand sb-vm::move-arg))
+                        (sb-vm::load-store-two-words vop (vop-next vop)))
                    (setf vop (vop-next vop)))
                   (t
                    (funcall gen vop)))))))
 
-    ;; Truncate the final assembly code buffer to length
-    (let ((data (cdr (asmstream-code-section asmstream))))
-      (setf (cadr data) (subseq (cadr data) 0 (car data))))
+    (when *do-instcombine-pass*
+      #+x86-64
+      (sb-assem::combine-instructions (asmstream-code-section asmstream)))
 
+    ;; Jump tables precede the coverage mark bytes to simplify locating
+    ;; them in trans_code().
+    (emit-jump-tables)
+    ;; Todo: can we implement the flow-based aspect of coverage mark compression
+    ;; in IR2 instead of waiting until assembly generation?
     (coverage-mark-lowering-pass component asmstream)
-
     (emit-inline-constants)
-    (let* ((fun-table)
-           (end-text (gen-label))
-           (start-fun-table (gen-label))
-           (n-entries (length (ir2-component-entries (component-info component))))
-           (fun-table-section
-            (let ((section (sb!assem::make-section)))
-              ;; Create space for the simple-fun offset table:
-              ;;  1 uint32 * N simple-funs
-              ;;  1 uint16 for the number of simple-funs
-              (emit section
-                    end-text `(.align 2) start-fun-table
-                    `(.skip ,(+ (* n-entries 4) 2)))
-              section))
-           (segment
-            (assemble-sections
-               (make-segment :header-skew
-                             (if (and (= code-boxed-words-align 1)
-                                      (oddp (length (ir2-component-constants
-                                                     (component-info component)))))
-                                 sb!vm:n-word-bytes
-                                 0)
-                             :run-scheduler (default-segment-run-scheduler)
-                             :inst-hook (default-segment-inst-hook))
-               (asmstream-data-section asmstream)
-               (asmstream-code-section asmstream)
-               (asmstream-elsewhere-section asmstream)
-               fun-table-section))
-           (octets
-            (finalize-segment segment))
-           (index
-            (label-position start-fun-table)))
-      (flet ((store-ub16 (index val)
-                 (multiple-value-bind (b0 b1)
-                     #!+little-endian
-                     (values (ldb (byte 8 0) val) (ldb (byte 8 8) val))
-                     #!+big-endian
-                     (values (ldb (byte 8 8) val) (ldb (byte 8 0) val))
-                   (setf (aref octets (+ index 0)) b0
-                         (aref octets (+ index 1)) b1)))
-             (store-ub32 (index val)
-                 (multiple-value-bind (b0 b1 b2 b3)
-                     #!+little-endian
-                     (values (ldb (byte 8  0) val) (ldb (byte 8  8) val)
-                             (ldb (byte 8 16) val) (ldb (byte 8 24) val))
-                     #!+big-endian
-                     (values (ldb (byte 8 24) val) (ldb (byte 8 16) val)
-                             (ldb (byte 8  8) val) (ldb (byte 8  0) val))
-                   (setf (aref octets (+ index 0)) b0
-                         (aref octets (+ index 1)) b1
-                         (aref octets (+ index 2)) b2
-                         (aref octets (+ index 3)) b3))))
-        (let ((padding (- index (label-position end-text))))
-          (unless (and (typep n-entries '(unsigned-byte 12))
-                       (typep padding '(unsigned-byte 4)))
-            (bug "Oversized code component?"))
-          ;; Assert that we are aligned for storing uint32_t
-          (aver (not (logtest index #b11)))
-          (dolist (entry (sb!c::ir2-component-entries
-                          (component-info component)))
-            (let ((val (label-position (entry-info-offset entry))))
-              (push val fun-table)
-              (store-ub32 index val)
-              (incf index 4)))
-          (store-ub16 index (logior (ash n-entries 4) padding)))
-        (aver (= (+ index 2) (length octets))))
-      (values segment (label-position end-text) fun-table
-              (asmstream-elsewhere-label asmstream)
-              (sb!assem::segment-fixup-notes segment)))))
+
+    (let* ((info (component-info component))
+           (simple-fun-labels
+            (mapcar #'entry-info-offset (ir2-component-entries info)))
+           (n-boxed (length (ir2-component-constants info)))
+           ;; Skew is either 0 or N-WORD-BYTES depending on whether the boxed
+           ;; header length is even or odd
+           (skew (if (and (= code-boxed-words-align 1) (oddp n-boxed))
+                     sb-vm:n-word-bytes
+                     0)))
+      (multiple-value-bind (segment text-length fixup-notes fun-table)
+          (assemble-sections asmstream
+                             simple-fun-labels
+                             (make-segment :header-skew skew
+                                           :run-scheduler (default-segment-run-scheduler)))
+        (values segment text-length fun-table
+                (asmstream-elsewhere-label asmstream) fixup-notes)))))
 
 (defun label-elsewhere-p (label-or-posn kind)
   (let ((elsewhere (label-position *elsewhere-label*))
@@ -332,58 +370,51 @@
         (< elsewhere label)
         (<= elsewhere label))))
 
-#!+inline-constants
-(defun register-inline-constant (&rest constant-descriptor)
-  (declare (dynamic-extent constant-descriptor))
-  (let ((asmstream *asmstream*)
-        (constant (sb!vm:canonicalize-inline-constant constant-descriptor)))
-    (ensure-gethash
-     constant
-     (asmstream-constant-table asmstream)
-     (multiple-value-bind (label value) (sb!vm:inline-constant-value constant)
-       (vector-push-extend (cons constant label)
-                           (asmstream-constant-vector asmstream))
-       value))))
-
 ;;; Translate .COVERAGE-MARK pseudo-op into machine assembly language,
 ;;; combining any number of consecutive operations with no intervening
 ;;; control flow into a single operation.
+;;; FIXME: this pass runs even if no coverage instrumentation was generated.
 (defun coverage-mark-lowering-pass (component asmstream)
   (declare (ignorable component asmstream))
-  #!+(or x86-64 x86)
+  #+(or x86-64 x86)
   (let ((label (gen-label))
         ;; vector of lists of original source paths covered
         (src-paths (make-array 10 :fill-pointer 0))
-        (previous-mark)
-        (section (asmstream-code-section asmstream)))
-    (dolist (buffer (reverse (cddr section)))
-      (dotimes (i (length buffer))
-        (let ((item (svref buffer i)))
-          (typecase item
-           (label
-            (when (label-usedp item) ; control can transfer to here
-              (setq previous-mark nil)))
-           (function ; this can do anything, who knows
-            (setq previous-mark nil))
-           (t
-            (let ((mnemonic (first item)))
-              (cond ((branch-opcode-p mnemonic) ; control flow kills mark combining
-                     (setq previous-mark nil))
-                    ((eq mnemonic '.coverage-mark)
-                     (let ((path (second item)))
-                       (cond ((not previous-mark) ; record a new path
-                              (let ((mark-index
-                                     (vector-push-extend (list path) src-paths)))
-                                ;; have the backend lower it into a real instruction
-                                (replace-coverage-instruction buffer i label mark-index))
-                              (setq previous-mark t))
-                             (t ; record that the already-emitted mark pertains
-                                ; to an additional source path
-                              (push path (elt src-paths (1- (fill-pointer src-paths))))
-                              ;; turn this line into a (virtual) no-op
-                              (rplaca item '.comment))))))))))))
+        (previous-mark))
+    (do ((statement (stmt-next (section-start (asmstream-code-section asmstream)))
+                    (stmt-next statement)))
+        ((null statement))
+      (dolist (item (ensure-list (stmt-labels statement)))
+        (when (label-usedp item) ; control can transfer to here
+          (setq previous-mark nil)))
+      (let ((mnemonic (stmt-mnemonic statement)))
+        (typecase mnemonic
+         (function ; this can do anything, who knows
+          (setq previous-mark nil))
+         (string) ; a comment can be ignored
+         (t
+          (cond ((branch-opcode-p mnemonic) ; control flow kills mark combining
+                 (setq previous-mark nil))
+                ((eq mnemonic '.coverage-mark)
+                 (let ((path (car (stmt-operands statement))))
+                   (cond ((not previous-mark) ; record a new path
+                          (let ((mark-index
+                                 (vector-push-extend (list path) src-paths)))
+                            ;; have the backend lower it into a real instruction
+                            (replace-coverage-instruction statement label mark-index))
+                            (setq previous-mark statement))
+                           (t ; record that the already-emitted mark pertains
+                            ;; to an additional source path
+                            (push path (elt src-paths (1- (fill-pointer src-paths))))
+                            ;; Clobber this statement. Do not try to remove it and
+                            ;; combine its labels into the previous statement.
+                            ;; Doing that could move a label into an earlier IR2 block
+                            ;; which crashes when dumping locations.
+                            (setf (stmt-mnemonic statement) nil
+                                  (stmt-operands statement) nil))))))))))
+
     ;; Allocate space in the data section for coverage marks
-    (let ((mark-index(length src-paths)))
+    (let ((mark-index (length src-paths)))
       (when (plusp mark-index)
         (setf (label-usedp label) t)
         (let ((v (ir2-component-constants (component-info component))))
