@@ -16,7 +16,7 @@
 ;;;; probably be good to rename this file to "call-type.lisp" or
 ;;;; "ir1-type.lisp" or something.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (declaim (type (or function null) *lossage-fun* *unwinnage-fun* *ctype-test-fun*))
 
@@ -34,8 +34,9 @@
 ;;; the function that we use for type checking. The derived type is
 ;;; its first argument and the type we are testing against is its
 ;;; second argument. The function should return values like CSUBTYPEP.
+;;; Can be NIL when no type testing is needed. (When ir1 converting,
+;;; as opposed to checking whether an ir1-transform is applicable.)
 (defvar *ctype-test-fun*)
-;;; FIXME: Why is this a variable? Explain.
 
 ;;; *LOSSAGE-DETECTED* is set when a "definite incompatibility" is
 ;;; detected. *UNWINNAGE-DETECTED* is set when we can't tell whether the
@@ -56,7 +57,6 @@
 (defvar *unwinnage-detected*)
 
 ;;; Signal a warning if appropriate and set *FOO-DETECTED*.
-(declaim (ftype (function (string &rest t) (values)) note-lossage note-unwinnage))
 (defun note-lossage (format-string &rest format-args)
   (setq *lossage-detected* t)
   (when *lossage-fun*
@@ -70,15 +70,6 @@
 
 
 ;;;; stuff for checking a call against a function type
-;;;;
-;;;; FIXME: This is stuff to look at when I get around to fixing
-;;;; function type inference and declarations.
-
-;;; A dummy version of SUBTYPEP useful when we want a functional like
-;;; SUBTYPEP that always returns true.
-(defun always-subtypep (type1 type2)
-  (declare (ignore type1 type2))
-  (values t t))
 
 ;;; Determine whether a use of a function is consistent with its type.
 ;;; These values are returned:
@@ -121,31 +112,16 @@
                (optional (fun-type-optional type))
                (max-args (+ min-args (length optional)))
                (rest (fun-type-rest type))
-               (keyp (fun-type-keyp type)))
+               (keyp (fun-type-keyp type))
+               (fun (combination-fun call))
+               (caller (loop for annotation in (lvar-annotations fun)
+                             when (typep annotation 'lvar-function-designator-annotation)
+                             do (setf *compiler-error-context* annotation)
+                                (return (lvar-function-designator-annotation-caller annotation)))))
           (cond
-            ((fun-type-wild-args type)
-             (loop for arg in args
-                   and i from 1
-                   do (check-arg-type arg *universal-type* i)))
-            ((not (or optional keyp rest))
-             (if (/= nargs min-args)
-                 (note-lossage
-                  "The function was called with ~R argument~:P, but wants exactly ~R."
-                  nargs min-args)
-                 (check-fixed-and-rest args required nil)))
-            ((< nargs min-args)
-             (note-lossage
-              "The function was called with ~R argument~:P, but wants at least ~R."
-              nargs min-args))
-            ((<= nargs max-args)
-             (check-fixed-and-rest args (append required optional) rest))
-            ((not (or keyp rest))
-             (note-lossage
-              "The function was called with ~R argument~:P, but wants at most ~R."
-              nargs max-args))
-            ((and keyp (oddp (- nargs max-args)))
-             (note-lossage
-              "The function has an odd number of arguments in the keyword portion."))
+            ((report-arg-count-mismatch (nth-value 1 (lvar-fun-type fun))
+                                        caller type nargs nil
+                                        :lossage-fun #'note-lossage))
             (t
              (check-fixed-and-rest args (append required optional) rest)
              (when (and keyp
@@ -176,12 +152,14 @@
               and i from 1
               do (check-arg-type arg *wild-type* i)))
     (awhen (lvar-fun-name (combination-fun call) t)
-      (let ((type (info :function :type it)))
-        (validate-test-and-test-not call)
-        ;; One more check for structure constructors:
-        (when (typep type 'defstruct-description)
-          (awhen (assq it (dd-constructors type))
-            (check-structure-constructor-call call type (cdr it))))))
+      (validate-test-and-test-not call)
+      (let ((xform (info :function :source-transform it)))
+        ;; One more check for structure constructors, because satisfying the
+        ;; ftype is not sufficient to ensure that slots get valid defaults.
+        (when (typep xform '(cons defstruct-description (eql :constructor)))
+          (let ((dd (car xform)))
+            (awhen (assq it (dd-constructors dd))
+              (check-structure-constructor-call call dd (cdr it)))))))
     (cond (*lossage-detected* (values nil t unknown-keys))
           (*unwinnage-detected* (values nil nil unknown-keys))
           (t (values t t unknown-keys)))))
@@ -229,12 +207,12 @@
                                   (dsd-default slot))))
                 ;; Return T if value-form definitely does not satisfy
                 ;; the type-check for DSD. Return NIL if we can't decide.
-                (when (if (sb!xc:constantp initform)
-                          (not (sb!xc:typep (constant-form-value initform)
+                (when (if (constantp initform)
+                          (not (sb-xc:typep (constant-form-value initform)
                                             (dsd-type slot)))
                           ;; Find uses of nil-returning functions as defaults,
                           ;; like ERROR and MISSING-ARG.
-                          (and (sb!kernel::dd-null-lexenv-p dd)
+                          (and (sb-kernel::dd-null-lexenv-p dd)
                                (listp initform)
                                (let ((f (car initform)))
                                  ;; Don't examine :function :type of macros!
@@ -256,21 +234,19 @@ and no value was provided for it." name))))))))))
 (defun check-arg-type (lvar type n)
   (declare (type lvar lvar) (type ctype type) (type index n))
   (cond
-   ((not (constant-type-p type))
-    (let ((ctype (lvar-type lvar)))
-      (multiple-value-bind (int win) (funcall *ctype-test-fun* ctype type)
-        (cond ((not win)
-               (note-unwinnage "can't tell whether the ~:R argument is a ~S"
-                               n (type-specifier type))
-               nil)
-              ((not int)
-               (note-lossage "The ~:R argument is a ~S, not a ~S."
-                             n (type-specifier ctype) (type-specifier type))
-               nil)
-              ((eq ctype *empty-type*)
-               (note-unwinnage "The ~:R argument never returns a value." n)
-               nil)
-              (t t)))))
+    ((not *ctype-test-fun*))
+    ((not (constant-type-p type))
+     (let* ((ctype (lvar-type lvar))
+            (int (funcall *ctype-test-fun* ctype type)))
+       (cond ((not int)
+              (unless (type= ctype (specifier-type '(member dummy)))
+                (note-lossage "The ~:R argument is a ~S, not a ~S."
+                              n (type-specifier ctype) (type-specifier type)))
+              nil)
+             ((eq ctype *empty-type*)
+              (note-unwinnage "The ~:R argument never returns a value." n)
+              nil)
+             (t t))))
     ((not (constant-lvar-p lvar))
      (note-unwinnage "The ~:R argument is not a constant." n)
      nil)
@@ -321,7 +297,9 @@ and no value was provided for it." name))))))))))
       (let ((k (first key))
             (v (second key)))
         (cond
-          ((not (check-arg-type k (specifier-type 'symbol) n)))
+          ((not (types-equal-or-intersect (lvar-type k) (specifier-type 'symbol)))
+           (note-lossage "The ~:R argument of type ~s cannot be used as a keyword."
+                         n (type-specifier (lvar-type k))))
           ((not (constant-lvar-p k))
            (setf unknown-keys t)
            ;; An unknown key may turn out to be :ALLOW-OTHER-KEYS at runtime,
@@ -393,9 +371,11 @@ and no value was provided for it." name))))))))))
            :keywords (keys)
            :keyp (optional-dispatch-keyp functional)
            :allowp (optional-dispatch-allowp functional)
-           :returns (tail-set-type
-                     (lambda-tail-set
-                      (optional-dispatch-main-entry functional))))))))
+           :returns (let ((tail-set (lambda-tail-set
+                                     (optional-dispatch-main-entry functional))))
+                      (if tail-set
+                          (tail-set-type tail-set)
+                          *wild-type*)))))))
 
 ;;;; approximate function types
 ;;;;
@@ -412,10 +392,10 @@ and no value was provided for it." name))))))))))
 (defstruct (approximate-fun-type (:copier nil))
   ;; the smallest and largest numbers of arguments that this function
   ;; has been called with.
-  (min-args sb!xc:call-arguments-limit
-            :type (integer 0 #.sb!xc:call-arguments-limit))
+  (min-args call-arguments-limit
+            :type (integer 0 #.call-arguments-limit))
   (max-args 0
-            :type (integer 0 #.sb!xc:call-arguments-limit))
+            :type (integer 0 #.call-arguments-limit))
   ;; a list of lists of the all the types that have been used in each
   ;; argument position
   (types () :type list)
@@ -433,7 +413,7 @@ and no value was provided for it." name))))))))))
   ;; The position at which this keyword appeared. 0 if it appeared as the
   ;; first argument, etc.
   (position (missing-arg)
-            :type (integer 0 #.sb!xc:call-arguments-limit))
+            :type (integer 0 #.call-arguments-limit))
   ;; a list of all the argument types that have been used with this keyword
   (types nil :type list)
   ;; true if this keyword has appeared only in calls with an obvious
@@ -579,23 +559,24 @@ and no value was provided for it." name))))))))))
 (declaim (ftype (function (list ctype string &rest t) (values))
                 check-approximate-arg-type))
 (defun check-approximate-arg-type (call-types decl-type context &rest args)
-  (let ((losers *empty-type*))
-    (dolist (ctype call-types)
-      (multiple-value-bind (int win) (funcall *ctype-test-fun* ctype decl-type)
-        (cond
-         ((not win)
-          (note-unwinnage "can't tell whether previous ~? ~
+  (when *ctype-test-fun*
+    (let ((losers *empty-type*))
+      (dolist (ctype call-types)
+        (multiple-value-bind (int win) (funcall *ctype-test-fun* ctype decl-type)
+          (cond
+            ((not win)
+             (note-unwinnage "can't tell whether previous ~? ~
                            argument type ~S is a ~S"
-                          context
-                          args
-                          (type-specifier ctype)
-                          (type-specifier decl-type)))
-         ((not int)
-          (setq losers (type-union ctype losers))))))
+                             context
+                             args
+                             (type-specifier ctype)
+                             (type-specifier decl-type)))
+            ((not int)
+             (setq losers (type-union ctype losers))))))
 
-    (unless (eq losers *empty-type*)
-      (note-lossage "~:(~?~) argument should be a ~S but was a ~S in a previous call."
-                    context args (type-specifier decl-type) (type-specifier losers))))
+      (unless (eq losers *empty-type*)
+        (note-lossage "~:(~?~) argument should be a ~S but was a ~S in a previous call."
+                      context args (type-specifier decl-type) (type-specifier losers)))))
   (values))
 
 ;;; Check the types of each manifest keyword that appears in a keyword
@@ -733,7 +714,7 @@ and no value was provided for it." name))))))))))
            ((lambda-var-arg-info arg)
             (let* ((info (lambda-var-arg-info arg))
                    (default (arg-info-default info))
-                   (def-type (when (sb!xc:constantp default)
+                   (def-type (when (constantp default)
                                (ctype-of (constant-form-value default)))))
               (ecase (arg-info-kind info)
                 (:keyword
@@ -859,11 +840,10 @@ and no value was provided for it." name))))))))))
                         (when (and unwinnage-fun
                                    (not (csubtypep (leaf-type var) type)))
                           (funcall unwinnage-fun
-                                   #.(#+sb-xc sb!impl::!xc-preprocess-format-control
-                                      #-sb-xc identity
+                                   (sb-format:tokens
                                       "Assignment to argument: ~S~%  ~
                                        prevents use of assertion from function ~
-                                       type ~A:~% ~/sb!impl:print-type/~%")
+                                       type ~A:~% ~/sb-impl:print-type/~%")
                                    (leaf-debug-name var) where type)))
                        ((and (listp really-assert) ; (:NOT . ,vars)
                              (member (lambda-var-%source-name var)
@@ -879,9 +859,9 @@ and no value was provided for it." name))))))))))
                                                       type-returns)))
            (note-lossage
             "The result type from ~A:~%  ~
-             ~/sb!impl:print-type/~@
+             ~/sb-impl:print-type/~@
              conflicts with the definition's result type:~%  ~
-             ~/sb!impl:print-type/"
+             ~/sb-impl:print-type/"
             where type-returns dtype)
            nil)
           (t
@@ -903,7 +883,7 @@ and no value was provided for it." name))))))))))
         (explicit-check (getf (functional-plist fun) 'explicit-check)))
     (if (eq where :declared)
         (let ((type
-               (massage-global-definition-type (proclaimed-ftype name) fun)))
+               (massage-global-definition-type (global-ftype name) fun)))
           (setf (leaf-type fun) type)
           (assert-definition-type
            fun type
@@ -939,9 +919,10 @@ and no value was provided for it." name))))))))))
 
 ;;; Call FUN with (arg-lvar arg-type lvars &optional annotation)
 (defun map-combination-args-and-types (fun call &optional info
-                                                          unknown-keys-fun)
+                                                          unknown-keys-fun
+                                                          declared-only)
   (declare (type function fun) (type combination call))
-  (binding* ((type (lvar-fun-type (combination-fun call)))
+  (binding* ((type (lvar-fun-type (combination-fun call) declared-only declared-only))
              (nil (fun-type-p type) :exit-if-null)
              (annotation (and info
                               (fun-info-annotation info)))
@@ -1011,7 +992,7 @@ and no value was provided for it." name))))))))))
 (defun apply-type-annotation (fun-name arg type lvars policy &optional annotation)
   (case (car annotation)
     (function-designator
-     (assert-function-designator fun-name lvars arg (cdr annotation))
+     (assert-function-designator fun-name lvars arg (cdr annotation) policy)
      t)
     (t
      (case (car annotation)
@@ -1072,7 +1053,9 @@ and no value was provided for it." name))))))))))
                     (not trusted))
                (reoptimize-lvar arg)))
            call
-           info))))
+           info
+           nil
+           t))))
   (values))
 
 ;;;; FIXME: Move to some other file.
@@ -1083,13 +1066,13 @@ and no value was provided for it." name))))))))))
       (let ((sources (lvar-all-sources tag)))
         (if (singleton-p sources)
             (compiler-style-warn
-              "~@<Using ~S of type ~/sb!impl:print-type/ as ~
+              "~@<Using ~S of type ~/sb-impl:print-type/ as ~
                a catch tag (which tends to be unportable because THROW ~
                and CATCH use EQ comparison)~@:>"
              (first sources) (lvar-type tag))
             (compiler-style-warn
               "~@<Using ~{~S~^~#[~; or ~:;, ~]~} in ~S of type ~
-               ~/sb!impl:print-type/ as a catch tag (which tends to be ~
+               ~/sb-impl:print-type/ as a catch tag (which tends to be ~
                unportable because THROW and CATCH use EQ comparison)~@:>"
              (rest sources) (first sources) (lvar-type tag)))))))
 
@@ -1102,7 +1085,7 @@ and no value was provided for it." name))))))))))
                 :format-control
                 "~@<Values ~2I~_[~{~S~^ ~}] ~I~_from ~S in~_~A ~
                    ~I~_is not of the declared return type of the function ~
-                   ~2I~_~/sb!impl:print-type-specifier/.~:>"
+                   ~2I~_~/sb-impl:print-type-specifier/.~:>"
                 :format-arguments (list values
                                         (first detail) code-context
                                         atype)))
@@ -1114,7 +1097,7 @@ and no value was provided for it." name))))))))))
                     :format-control
                     "~@<Values ~2I~_[~{~S~^ ~}] ~I~_from ~S in~_~A ~
                      ~I~_is not of type ~
-                    ~2I~_~/sb!impl:print-type-specifier/.~:>"
+                    ~2I~_~/sb-impl:print-type-specifier/.~:>"
                     :format-arguments (list values
                                             (first detail) code-context
                                             atype))
@@ -1125,7 +1108,7 @@ and no value was provided for it." name))))))))))
                     "~@<Values ~2I~_[~{~S~^ ~}] ~
                    ~I~_from ~2I~_~{~S~^~#[~; or ~:;, ~]~} ~
                    ~I~_of ~2I~_~S ~I~_in~_~A ~I~_is not of type ~
-                   ~2I~_~/sb!impl:print-type-specifier/.~:>"
+                   ~2I~_~/sb-impl:print-type-specifier/.~:>"
                     :format-arguments (list values
                                             (rest detail) (first detail)
                                             code-context
@@ -1136,7 +1119,7 @@ and no value was provided for it." name))))))))))
                 :expected-type atype
                 :format-control
                 "~@<Value of ~S in ~_~A ~I~_is ~2I~_~S, ~
-                   ~I~_not a ~2I~_~/sb!impl:print-type-specifier/.~:@>"
+                   ~I~_not a ~2I~_~/sb-impl:print-type-specifier/.~:@>"
                 :format-arguments (list (car detail) code-context
                                         (car values)
                                         atype)))
@@ -1147,7 +1130,7 @@ and no value was provided for it." name))))))))))
                 :format-control
                 "~@<Value from ~2I~_~{~S~^~#[~; or ~:;, ~]~} ~
                    ~I~_of ~2I~_~S ~I~_in~_~A ~I~_is ~2I~_~S, ~
-                   ~I~_not a ~2I~_~/sb!impl:print-type-specifier/.~:@>"
+                   ~I~_not a ~2I~_~/sb-impl:print-type-specifier/.~:@>"
                 :format-arguments (list (rest detail) (first detail) code-context
                                         (car values)
                                         atype)))))
@@ -1162,7 +1145,7 @@ and no value was provided for it." name))))))))))
                  :format-control
                  "~@<Derived type of ~S is ~2I~_~S, ~
                     ~I~_conflicting with the declared function return type ~
-                    ~2I~_~/sb!impl:print-type-specifier/.~@:>"
+                    ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                  :format-arguments (list detail dtype atype)))
           ((singleton-p detail)
            (let ((detail (first detail)))
@@ -1171,22 +1154,22 @@ and no value was provided for it." name))))))))))
                        :format-control
                        "~@<Constant ~2I~_~S ~Iconflicts with its ~
                             asserted type ~
-                            ~2I~_~/sb!impl::print-type-specifier/.~@:>"
+                            ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                        :format-arguments (list (constant-form-value detail) atype))
                  (warn condition
                        :format-control
                        "~@<Derived type of ~S is ~2I~_~S, ~
                             ~I~_conflicting with its asserted type ~
-                            ~2I~_~/sb!impl:print-type-specifier/.~@:>"
+                            ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                        :format-arguments (list detail dtype atype)))))
           (t
            (warn condition
                  :format-control
                  "~@<Derived type of ~2I~_~{~S~^~#[~; and ~:;, ~
                       ~]~} ~I~_in ~2I~_~S ~I~_is ~
-                      ~2I~_~/sb!impl:print-type-specifier/, ~
+                      ~2I~_~/sb-impl:print-type-specifier/, ~
                       ~I~_conflicting with their asserted type ~
-                      ~2I~_~/sb!impl:print-type-specifier/.~@:>"
+                      ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                  :format-arguments (list (rest detail) (first detail)
                                          dtype atype))))))
 
