@@ -75,8 +75,31 @@
 (defun ir1-transform-type-predicate (object type node)
   (declare (type lvar object) (type ctype type))
   (let ((otype (lvar-type object)))
-    (flet ((tricky ()
-             (cond ((typep type 'alien-type-type)
+    (cond ((not (types-equal-or-intersect otype type))
+           (return-from ir1-transform-type-predicate nil))
+          ((csubtypep otype type)
+           (return-from ir1-transform-type-predicate t))
+          ((eq type *empty-type*)
+           (return-from ir1-transform-type-predicate nil)))
+    (let ((intersect (type-intersection2 type otype)))
+      ;; I guess the theory here is that an intersection type
+      ;; is never a singleton, because if we could see that it was
+      ;; a singleton, it wouldn't be an intersection.
+      (when (and intersect (not (intersection-type-p intersect)))
+        (multiple-value-bind (constantp value) (type-singleton-p intersect)
+          (when constantp
+            (return-from ir1-transform-type-predicate `(eql object ',value)))))
+      ;; If the object type is known to be (OR NULL <type>),
+      ;; it is almost always cheaper to test for not EQ to NIL.
+      ;; There is one exception:
+      ;;  - FIXNUMP is possibly cheapear than comparison to NIL, or definitely
+      ;;    not worse. For x86, NIL is a 4-byte immediate operand,
+      ;;    for lack of a null-tn register. FIXNUM-TAG-MASK is only 1 byte.
+      (when (type= otype (type-union (specifier-type 'null) type))
+        (let ((difference (type-difference type (specifier-type 'null))))
+          (unless (type= difference (specifier-type 'fixnum))
+            (return-from  ir1-transform-type-predicate `(not (null object))))))
+      (cond ((typep type 'alien-type-type)
                     ;; We don't transform alien type tests until here, because
                     ;; once we do that the rest of the type system can no longer
                     ;; reason about them properly -- so we'd miss out on type
@@ -86,48 +109,8 @@
                       ;; If it's a lisp-rep-type, the CTYPE should be one already.
                       (aver (not (compute-lisp-rep-type alien-type)))
                       `(sb-alien::alien-value-typep object ',alien-type)))
-                   #+(vop-translates sb-int:fixnump-instance-ref)
-                   ((and (type= type (specifier-type 'fixnum))
-                         (let ((use (lvar-uses object)))
-                           (and (combination-p use)
-                                (almost-immediately-used-p object use)
-                                (or (and (eq (lvar-fun-name (combination-fun use))
-                                             '%instance-ref)
-                                         (constant-lvar-p
-                                          (second (combination-args use))))
-                                    (member (lvar-fun-name (combination-fun use))
-                                            '(car cdr))))))
-                    ;; This is a disturbing trend, but it's the best way to
-                    ;; combine instructions in the compiler as it is
-                    ;; (as opposed to the compiler as we wish it would be).
-                    (case (lvar-fun-name (combination-fun (lvar-uses object)))
-                     (%instance-ref
-                      (splice-fun-args object '%instance-ref 2)
-                      `(lambda (obj i) (fixnump-instance-ref obj i)))
-                     (car
-                      (splice-fun-args object 'car 1)
-                      `(lambda (obj) (fixnump-car obj)))
-                     (cdr
-                      (splice-fun-args object 'cdr 1)
-                      `(lambda (obj) (fixnump-cdr obj)))))
-                   (t
-                    (give-up-ir1-transform)))))
-      (cond ((not (types-equal-or-intersect otype type))
-            nil)
-           ((csubtypep otype type)
-            t)
-           ((eq type *empty-type*)
-            nil)
-           (t
-            (let ((intersect (type-intersection2 type otype)))
-              (when (or (not intersect)
-                        (intersection-type-p intersect))
-                (tricky))
-              (multiple-value-bind (constantp value)
-                  (type-singleton-p intersect)
-                (if constantp
-                    `(eql object ',value)
-                    (tricky)))))))))
+            (t
+             (give-up-ir1-transform))))))
 
 ;;; Flush %TYPEP tests whose result is known at compile time.
 (deftransform %typep ((object type) * * :node node)
@@ -327,18 +310,29 @@
                                type)))
                  (rational 'rational)
                  (float (or (numeric-type-format type) 'float))
-                 ((nil) 'real))))
+                 ((nil) 'real)))
+         (low (numeric-type-low type))
+         (high (numeric-type-high type)))
     (ecase (numeric-type-complexp type)
       (:real
-       (cond #+(or x86 x86-64 arm arm64) ;; Not implemented elsewhere yet
-             ((and
-               (eql (numeric-type-class type) 'integer)
-               (or (eql (numeric-type-low type) 0)
-                   (eql (numeric-type-low type) 1))
-               (fixnump (numeric-type-high type)))
+       (cond ((and (eql (numeric-type-class type) 'integer)
+                   (and (fixnump low)
+                        (fixnump high)
+                        (<= (1+ (- high low)) 2)))
+              ;; The fixnum-mod-p case is worse than just EQ testing with
+              ;; only 2 values in the range. (INTEGER 1 2) would have become
+              ;;   (and (not (eq x 0)) (fixnump x) (not (> x 2))).
+              ;; If exactly 1 value, it should have been picked off by TYPE-SINGLETON-P
+              ;; in %SOURCE-TRANSFORM-TYPEP, but even if it wasn't,
+              ;; the OR will drop out due to constraint propagation.
+              `(or (eq ,object ,low) (eq ,object ,high)))
+             #+(or x86 x86-64 arm arm64) ;; Not implemented elsewhere yet
+             ((and (eql (numeric-type-class type) 'integer)
+                   (or (eql low 0) (eql low 1))
+                   (fixnump (numeric-type-high type)))
               (let ((mod-p
                       `(fixnum-mod-p ,object ,(numeric-type-high type))))
-                (if (eql (numeric-type-low type) 1)
+                (if (eql low 1)
                     `(and (not (eq ,object 0))
                           ,mod-p)
                     mod-p)))
@@ -601,9 +595,9 @@
                #+(and sb-unicode (or x86-64 arm64))
                ((= (cdar pairs) (1- base-char-code-limit))
                 `(base-char-p ,object))
-               ((= (cdar pairs) (1- sb-xc:char-code-limit))
+               ((= (cdar pairs) (1- char-code-limit))
                 `(characterp ,object))))
-        (let ((n-code (gensym "CODE")))
+        (let ((n-code (sb-xc:gensym "CODE")))
           `(and (characterp ,object)
                 (let ((,n-code (sb-xc:char-code ,object)))
                   (or
@@ -615,7 +609,7 @@
 (defun source-transform-simd-pack-typep (object type)
   (if (type= type (specifier-type 'simd-pack))
       `(simd-pack-p ,object)
-      (let ((n-tag (gensym "TAG")))
+      (let ((n-tag (sb-xc:gensym "TAG")))
         `(and
           (simd-pack-p ,object)
           (let ((,n-tag (%simd-pack-tag ,object)))
@@ -628,7 +622,7 @@
 (defun source-transform-simd-pack-256-typep (object type)
   (if (type= type (specifier-type 'simd-pack-256))
       `(simd-pack-256-p ,object)
-      (let ((n-tag (gensym "TAG")))
+      (let ((n-tag (sb-xc:gensym "TAG")))
         `(and
           (simd-pack-256-p ,object)
           (let ((,n-tag (%simd-pack-256-tag ,object)))
@@ -671,10 +665,9 @@
       (cond ((cdr dims)
              (values `(,header-test
                        ,@(when (eq (array-type-dimensions stype) '*)
-                           #+x86-64
-                           `((%array-rank= ,obj ,(length dims)))
-                           #-x86-64
-                           `((= (%array-rank ,obj) ,(length dims))))
+                           (if (vop-existsp :translate %array-rank=)
+                               `((%array-rank= ,obj ,(length dims)))
+                               `((= (%array-rank ,obj) ,(length dims)))))
                        ,@(loop for d in dims
                                for i from 0
                                unless (eq '* d)
@@ -788,9 +781,7 @@
 ;;; flushed if the result is known at compile time. If not properly
 ;;; named, error. If sealed and has no subclasses, just test for
 ;;; layout-EQ. If a structure then test for layout-EQ and then a
-;;; general test based on layout-inherits. If safety is important,
-;;; then we also check whether the layout for the object is invalid
-;;; and signal an error if so. Otherwise, look up the indirect
+;;; general test based on layout-inherits. Otherwise, look up the indirect
 ;;; class-cell and call CLASS-CELL-TYPEP at runtime.
 (deftransform %instance-typep ((object spec) (* *) * :node node)
   (aver (constant-lvar-p spec))
@@ -814,63 +805,107 @@
        (delay-ir1-transform node :constraint)
        (transform-instance-typep class)))))
 
-;;; This transform contains more comments than code. I wish there were some way
-;;; to express it more simply.
-(defun transform-instance-typep (class)
-  (let* ((name (classoid-name class))
-          (layout (let ((res (info :type :compiler-layout name)))
-                   (if (and res (not (layout-invalid res)))
-                       res
-                       nil))))
-       ;; FIXME: (TYPEP X 'ERROR) - or any condition - checks whether X
-       ;; has the lowtag of either an ordinary or funcallable instance.
-       ;; But you can not define a class that is both CONDITION and FUNCTION
-       ;; because CONDITION-CLASS and FUNCALLABLE-STANDARD-CLASS are
-       ;; incompatible metaclasses. Thus the type test is less efficient than
-       ;; could be, since fun-pointer-lowtag can not occur in the "true" case.
+;;; Notice that there are some instance types for which it is almost impossible
+;;; to create. One such is SEQUENCE, viz: (make-instance 'sequence) =>
+;;;   "Cannot allocate an instance of #<BUILT-IN-CLASS SEQUENCE>."
+;;; We should not need to check for that, just the 'inherits' vector.
+;;; However, bootstrap code does a sleazy thing, making an instance of
+;;; the abstract base type which is impossible for user code to do.
+;;;
+;;; Preferably the prototype instance for SEQUENCE would be one that could
+;;; exist, so it would be a STANDARD-OBJECT and SEQUENCE. But it's not.
+;;; Hence we would have to check for a layout that no code using the documented
+;;; sequence API would ever see, just to get the boundary case right.
+;;; The for STREAM and FILE-STREAM.
+;;; But there was precedent for builtin class prototype instances
+;;; failing their type predicate, i.e. (TYPEP (CLASS-PROTOTYPE X) X) => NIL
+;;; which was fixed in git rev d60a6d30.
+;;; Also for what it's worth, some builtins use a prototype object that is strictly
+;;; deeper than layout of the named class because it is indeed the case that no
+;;; object's layout can ever be EQ to that of the ancestor.
+;;; e.g. a fixnum as representative of class REAL.
+;;; So in actual practice, you can't make something that is a pure STREAM, etc.
+(defvar *use-layout-ids* (or #+(or ppc64 x86 x86-64) t))
+(defun transform-instance-typep (classoid)
+  (binding*
+      ((name (classoid-name classoid))
+       (layout (let ((res (info :type :compiler-layout name)))
+                 (when (and res (not (layout-invalid res))) res)))
+       ((primtype-predicate slot-reader)
+        (cond ((csubtypep classoid (specifier-type 'funcallable-instance))
+               (values '(function-with-layout-p object)
+                       '(%fun-layout object)))
+              ((or (csubtypep classoid (specifier-type 'instance))
+                   (eq classoid (specifier-type 'logical-pathname))
+                   ;; CONDITION can't be a funcallable-instance
+                   (csubtypep classoid (specifier-type 'condition)))
+               (values '(%instancep object)
+                       '(%instance-layout object)))))
+       (get-layout-or-return-false
+        (if primtype-predicate
+            ;; Test just one of %INSTANCEP or %FUNCALLABLE-INSTANCE-P
+            `(if ,primtype-predicate ,slot-reader (return-from typep nil))
+            ;; But if we don't know which is will be, try both.
+            ;; This is less general than LAYOUT-OF,and therefore
+            ;; a little quicker to fail, because objects with
+            ;; {LIST|OTHER}-POINTER-LOWTAG can't possibly pass.
+            ;; It's a bit disappointing that STREAM uses this slower path,
+            ;; but some people think it should be possible to create
+            ;; funcallable streams. As a countermeasure, it would be possible to emit
+            ;; slightly better code in a vop.
+            `(cond ((%instancep object) (%instance-layout object))
+                   ((function-with-layout-p object) (%fun-layout object))
+                   (t (return-from typep nil)))))
+       (depthoid (if layout (layout-depthoid layout) -1))
+       (n-layout (make-symbol "LAYOUT"))
+       (n-inherits (make-symbol "INHERITS"))
+        ;; In order to efficiently perform the DEEPER-P test without this hack of using
+        ;; a vop (when available for non-risc machines), we'd have to do two things:
+        ;; - have instcombine combine the read and compare as one instruction
+        ;; - implement half-width structure slots
+        ;; Since both of those are not happening any time soon, ...
+       (deeper-p
+        (if (vop-existsp :translate layout-depthoid-gt)
+            `(layout-depthoid-gt ,n-layout ,depthoid)
+            `(> (layout-depthoid ,n-layout) ,depthoid)))
+       (nth-ancestor ; This is possibly unused (if no compile-time layout, or depthoid -1)
+        ;; Use DATA-VECTOR-REF directly, since that's what SVREF in SAFETY 0 will become.
+        `(locally (declare (optimize (safety 0)))
+           (data-vector-ref (layout-inherits ,n-layout) ,depthoid)))
+       (ancestor-layout-eq
+        `(eq ,nth-ancestor ,layout)))
 
-       ;; Otherwise transform the type test.
-       (binding* (((pred get-layout)
-                   (cond ((csubtypep class (specifier-type 'funcallable-instance))
-                          (values '(funcallable-instance-p object)
-                                  '(%funcallable-instance-layout object)))
-                         ((csubtypep class (specifier-type 'instance))
-                          (values '(%instancep object)
-                                  '(%instance-layout object)))))
-                  (get-layout-or-return-false
-                   (if pred
-                       ;; Test just one of %INSTANCEP or %FUNCALLABLE-INSTANCE-P
-                       `(if ,pred ,get-layout (return-from typep nil))
-                       ;; But if we don't know which is will be, try both.
-                       ;; This is less general than LAYOUT-OF,and therefore
-                       ;; a little quicker to fail, because objects with
-                       ;; {LIST|OTHER}-POINTER-LOWTAG can't possibly pass.
-                       `(cond ((%instancep object)
-                               (%instance-layout object))
-                              ((funcallable-instance-p object)
-                               (%funcallable-instance-layout object))
-                              (t (return-from typep nil)))))
-                  (n-layout (gensym)))
-         (cond
-           ;; It's possible to seal a STANDARD-CLASS, not just a STRUCTURE-CLASS,
-           ;; though probably extremely weird. Also the PRED should be set in
-           ;; that event, but it isn't.
-           ((and (eq (classoid-state class) :sealed) layout
-                 (or (not (classoid-subclasses class))
-                     (eql (hash-table-count (classoid-subclasses class))
-                          1)))
-            ;; Sealed and at most one subclass.
+    ;; Easiest case first: single bit test.
+    (cond ((member name '(condition pathname structure-object))
+           `(and (%instancep object)
+                 (logtest (layout-flags (%instance-layout object))
+                          ,(case name
+                             (condition +condition-layout-flag+)
+                             (pathname  +pathname-layout-flag+)
+                             (t         +structure-layout-flag+)))))
+
+          ;; Next easiest: Sealed and at most one subclass.
+          ;; However, it is better not to perform two IFs when a single one
+          ;; will suffice, so forbid any subclass when using layout-ids.
+          ((and layout
+                (eq (classoid-state classoid) :sealed)
+                (or (not (classoid-subclasses classoid))
+                    (and (not *use-layout-ids*)
+                         (eql (hash-table-count (classoid-subclasses classoid))
+                              1))))
+            ;; It's possible to seal a STANDARD-CLASS, not just a STRUCTURE-CLASS,
+            ;; though probably extremely weird. Also the PRED should be set in
+            ;; that event, but it isn't.
             ;; The crummy dual expressions for the same result are because
             ;; (BLOCK (RETURN ...)) seems to emit a forward branch in the
             ;; passing case, but AND emits a forward branch in the failing
             ;; case which I believe is the better choice.
-            (let ((other-layout (and
-                                 (classoid-subclasses class)
-                                 (dohash ((classoid layout)
-                                          (classoid-subclasses class)
-                                          :locked t)
-                                   (declare (ignore classoid))
-                                   (return layout)))))
+            (let ((other-layout (and (classoid-subclasses classoid)
+                                     (dohash ((classoid layout)
+                                              (classoid-subclasses classoid)
+                                              :locked t)
+                                             (declare (ignore classoid))
+                                             (return layout)))))
               (flet ((check-layout (layout-getter)
                        (cond (other-layout
                               ;; It's faster to compare two layouts than
@@ -878,129 +913,78 @@
                               `(let ((object-layout ,layout-getter))
                                  (or (eq object-layout ',layout)
                                      (eq object-layout ',other-layout))))
-                             ;; FIXME: not defined in time for make-host-1,
-                             ;; so the cross-compiler isn't getting this branch.
-                             #+(vop-named sb-vm::layout-eq)
-                             ((equal layout-getter '(%instance-layout object))
+                             ((and (vop-existsp :named sb-vm::layout-eq)
+                                   (equal layout-getter '(%instance-layout object)))
                               `(sb-vm::layout-eq object ',layout))
                              (t
                               `(eq ,layout-getter ',layout)))))
-                (if pred
-                    `(and ,pred ,(check-layout get-layout))
+                (if primtype-predicate
+                    `(and ,primtype-predicate ,(check-layout slot-reader))
                     `(block typep ,(check-layout get-layout-or-return-false))))))
 
-           ((and (typep class 'structure-classoid) layout)
+          ;; All other structure types
+          ((and (typep classoid 'structure-classoid) layout)
             ;; structure type tests; hierarchical layout depths
-            (let* ((depthoid (layout-depthoid layout))
-                   ;; If a structure is apparently an abstract base type,
-                   ;; having no constructor, then no instance layout should
-                   ;; be EQ to the classoid's layout. It is a slight win
-                   ;; to use the depth-based check first, then do the EQ check.
-                   ;; There is no loss in the case where both fail, and there
-                   ;; is a benefit in a passing case. Always try both though,
-                   ;; because (MAKE-INSTANCE 'x) works on any structure class.
-                   (abstract-base-p (awhen (layout-info layout)
-                                      (not (dd-constructors it))))
-                   (get-ancestor
-                     ;; Use DATA-VECTOR-REF directly, since that's what SVREF in
-                     ;; a SAFETY 0 lexenv will eventually be transformed to.
-                     ;; This can give a large compilation speedup, since
-                     ;; %INSTANCE-TYPEPs are frequently created during
-                     ;; GENERATE-TYPE-CHECKS, and the normal aref transformation
-                     ;; path is pretty heavy.
-                     `(locally (declare (optimize (safety 0)))
-                        (data-vector-ref (layout-inherits ,n-layout) ,depthoid)))
-                   (ancestor-layout-eq
-                     ;; Layouts are immediate constants in immobile space.
-                     ;; It would be far nicer if we had a pattern-matching pass
-                     ;; wherein the backend would recognize that
-                     ;; (eq (data-vector-ref ...) k) has a single instruction form,
-                     ;; but lacking that, force it into a single call
-                     ;; that a vop can translate.
-                     #+(and immobile-space x86-64)
-                     `(sb-vm::layout-inherits-ref-eq ; only implemented on x86-64
-                       (layout-inherits ,n-layout) ,depthoid ,layout)
-                     #-(and immobile-space x86-64)
-                     `(eq ,get-ancestor ,layout))
-                   (deeper-p
-                    #+(vop-translates sb-c::layout-depthoid-gt)
-                    `(layout-depthoid-gt ,n-layout ,depthoid)
-                    #-(vop-translates sb-c::layout-depthoid-gt)
-                    `(> (layout-depthoid ,n-layout) ,depthoid)))
-              (aver (equal pred '(%instancep object)))
-              (case name
-                (structure-object
-                 (return-from transform-instance-typep
-                   `(and (%instancep object)
-                         (logtest (layout-%bits (%instance-layout object))
-                                  +structure-layout-flag+))))
-                (pathname
-                 (return-from transform-instance-typep
-                   `(and (%instancep object)
-                         (logtest (layout-%bits (%instance-layout object))
-                                  +pathname-layout-flag+)))))
-              ;; For shallow hierarchies, we can avoid reading the 'inherits'
-              ;; because the layout has the ancestor layouts directly in it.
-              ;; Not even a depthoid check is needed.
-              ;; Since only layouts for structure types will have ancestors
-              ;; populated, and this transform case is only for structures,
-              ;; there can be no false positives. STREAM and CONDITION types
-              ;; have a depthoid>0, but are not structure-classoid-p.
-              `(and (%instancep object)
-                    (let ((,n-layout (%instance-layout object)))
-                      ;; we used to check for invalid layouts here,
-                      ;; but in fact that's both unnecessary and
-                      ;; wrong; it's unnecessary because structure
-                      ;; classes can't be redefined, and it's wrong
-                      ;; because it is quite legitimate to pass an
-                      ;; object with an invalid layout to a structure
-                      ;; type test.
-                      ,(let ((ancestor-slot (case depthoid
-                                             (2 'sb-kernel::layout-depth2-ancestor)
-                                             (3 'sb-kernel::layout-depth3-ancestor)
-                                             (4 'sb-kernel::layout-depth4-ancestor))))
-                         (if ancestor-slot
-                             (if abstract-base-p
-                                 `(or (eq (,ancestor-slot ,n-layout) ,layout)
-                                      (eq ,n-layout ,layout)) ; not likely
-                                 ;; Indifferent to order here. Might as well test
-                                 ;; for an exact match first.
-                                 `(or (eq ,n-layout ,layout)
-                                      (eq (,ancestor-slot ,n-layout) ,layout)))
-                             (if abstract-base-p
-                                 `(eq (if ,deeper-p ,get-ancestor ,n-layout) ,layout)
-                                 `(cond ((eq ,n-layout ,layout) t)
-                                        (,deeper-p ,ancestor-layout-eq)))))))))
-           ((and layout (>= (layout-depthoid layout) 0))
-            ;; hierarchical layout depths for other things (e.g.
-            ;; CONDITION, STREAM)
-            ;; The quasi-hierarchical types are abstract base types,
-            ;; so perform inheritance check first, and EQ second.
-            ;; Actually, since you can't make an abstract STREAM,
-            ;; maybe we should skip the EQ test? But you *can* make
-            ;; an instance of CONDITION for what it's worth.
-            ;; SEQUENCE is special-cased, but could be handled here.
-            (let* ((depthoid (layout-depthoid layout))
-                   (n-inherits (gensym))
-                   (guts
-                     `((when (layout-invalid ,n-layout)
-                         (setq ,n-layout (update-object-layout-or-invalid
-                                          object ',layout)))
-                       (let ((,n-inherits (layout-inherits
-                                           (truly-the layout ,n-layout))))
-                         (declare (optimize (safety 0)))
-                         (eq (if (> (vector-length ,n-inherits) ,depthoid)
-                                 (data-vector-ref ,n-inherits ,depthoid)
-                                 ,n-layout)
-                             ,layout)))))
-              (if pred
-                  `(and ,pred (let ((,n-layout ,get-layout)) ,@guts))
+            (aver (equal primtype-predicate '(%instancep object)))
+            ;; we used to check for invalid layouts here, but in fact that's both unnecessary and
+            ;; wrong; it's unnecessary because structure classes can't be redefined, and it's wrong
+            ;; because it is quite legitimate to pass an object with an invalid layout
+            ;; to a structure type test.
+            `(and (%instancep object)
+                    ;; If we allowed structure classes to be mixed in to standard-object,
+                    ;; this might have to change to consider object invalidation. Probably would
+                    ;; want to track structure classoids that would render this code inadmissible.
+                  (let ((,n-layout (%instance-layout object)))
+                    ,(if *use-layout-ids*
+                         (if (<= depthoid sb-kernel::layout-id-vector-fixed-capacity)
+                             `(%structure-is-a ,n-layout ,layout)
+                             `(and (layout-depthoid-ge ,n-layout ,depthoid)
+                                   (%structure-is-a ,n-layout ,layout)))
+                         ;; Distinguish between abstract base types - no DD-CONSTRUCTOR,
+                         ;; hence no direct instances - and everything else.
+                         (cond ((dd-constructors (layout-dd layout))
+                                `(cond ((eq ,n-layout ,layout) t)
+                                       (,deeper-p ,ancestor-layout-eq)))
+                               (t ; abstract base type deeper than optimized max.
+                                ;; Assume that no layout is EQ to the base layout,
+                                ;; and unconditionally fetch and dereference layout-inherits.
+                                `(eq (if ,deeper-p ,nth-ancestor ,n-layout) ,layout)))))))
+
+          ((> depthoid 0) ; fixed-depth ancestors of non-structure types: STREAM, FILE-STREAM,
+           ;; SEQUENCE; all are abstract base types.
+           ;; (I don't know why sometimes we just call SEQUENCEP and STREAMP. Policy?)
+            #+sb-xc-host (when (typep classoid 'static-classoid)
+                           ;; should have use :SEALED code above
+                           (bug "Non-frozen static classoids?"))
+            ;; There is no need to test the inheritance depth for depthoid 1 because INHERITS
+            ;; will contains at least #<LAYOUT for T> and a padding word if naught else.
+            ;; Though in actual fact it can't have just T, because users have no means to create
+            ;; types that are direct descendants of type T.
+            ;; However, we do have to avoid unsafely dereferencing inherits for FILE-STREAM
+            ;; which is at depth 4. An easy way to avoid this depth test would be requiring at
+            ;; least 5 elements in any INHERITS vector.  Alternatively, adding 1 bit each for
+            ;; SEQUENCE, STREAM, and FILE-STREAM to LAYOUT-FLAGS would suffice.
+            (let ((guts `(,@(unless (eq name 'condition)
+                              `((when (zerop (layout-clos-hash ,n-layout))
+                                 (setq ,n-layout
+                                  (truly-the layout
+                                             (update-object-layout-or-invalid
+                                              object ',layout))))))
+                          ,(if (eql depthoid 1)
+                               ancestor-layout-eq
+                               `(let ((,n-inherits (layout-inherits ,n-layout)))
+                                  (and (> (length ,n-inherits) ,depthoid)
+                                       (eq (data-vector-ref ,n-inherits ,depthoid)
+                                           ,layout)))))))
+
+              (if primtype-predicate
+                  `(and ,primtype-predicate (let ((,n-layout ,slot-reader)) ,@guts))
                   `(block typep
                      (let ((,n-layout ,get-layout-or-return-false)) ,@guts)))))
 
-           (t
+          (t
             `(classoid-cell-typep ',(find-classoid-cell name :create t)
-                                  object))))))
+                                  object)))))
 
 ;;; If the specifier argument is a quoted constant, then we consider
 ;;; converting into a simple predicate or other stuff. If the type is
@@ -1019,6 +1003,10 @@
   (let ((ctype (careful-specifier-type type)))
     (if ctype
         (or
+         ;; It's purely a waste of compiler resources to wait for IR1 to
+         ;; see these 2 edge cases that can be decided right now.
+         (cond ((eq ctype *universal-type*) t)
+               ((eq ctype *empty-type*) nil))
          (and (not (intersection-type-p ctype))
               (multiple-value-bind (constantp value) (type-singleton-p ctype)
                 (and constantp
@@ -1046,8 +1034,6 @@
            (args-type
             (compiler-warn "illegal type specifier for TYPEP: ~S" type)
             (return-from %source-transform-typep (values nil t)))
-           (t nil))
-         (typecase ctype
            (numeric-type
             (source-transform-numeric-typep object ctype))
            (classoid
@@ -1071,7 +1057,7 @@
 (defun source-transform-typep (object type)
   (when (typep type 'type-specifier)
     (check-deprecated-type type))
-  (let ((name (gensym "OBJECT")))
+  (let ((name (sb-xc:gensym "OBJECT")))
     (multiple-value-bind (transform error)
         (%source-transform-typep name type)
       (if error
@@ -1079,6 +1065,44 @@
           (values `(let ((,name ,object))
                      (%typep-wrapper ,transform ,name ',type)))))))
 
+;;; These things will be removed by the tree shaker, so no #+ needed.
+(defvar *interesting-types* nil)
+(defun involves-alien-p (ctype)
+  (sb-kernel::map-type
+   (lambda (type)
+     (when (alien-type-type-p type) (return-from involves-alien-p t)))
+   ctype))
+(defun dump/restore-interesting-types (op)
+  (declare (ignorable op))
+  #+collect-typep-regression-dataset
+  (ecase op
+   (write
+    (when *interesting-types*
+      (let ((list (sort (loop for k being each hash-key of *interesting-types* collect k)
+                        #'string< :key #'write-to-string)))
+        (with-open-file (f "interesting-types.lisp-expr" :direction :output
+                         :if-exists :supersede  :if-does-not-exist :create)
+          (let ((*package* #+sb-xc-host (find-package "XC-STRICT-CL")
+                           #-sb-xc-host #.(find-package "SB-KERNEL"))
+                (*print-pretty* nil)
+                (*print-length* nil)
+                (*print-level* nil)
+                (*print-readably* t))
+            (dolist (item list)
+              (write (uncross item) :stream f)
+              (terpri f)))))))
+   (read
+    (unless (hash-table-p *interesting-types*)
+      (setq *interesting-types* (make-hash-table :test 'equal :synchronized t)))
+    (with-open-file (f "interesting-types.lisp-expr" :if-does-not-exist nil)
+      (when f
+        (let ((*package* (find-package "SB-KERNEL")))
+          (loop (let ((expr (read f nil f)))
+                  (when (eq expr f) (return))
+                  (format t "Read ~a~%" expr)
+                  (setf (gethash expr *interesting-types*) t))))))
+    *interesting-types*)))
+
 (define-source-transform typep (object spec &optional env)
   ;; KLUDGE: It looks bad to only do this on explicitly quoted forms,
   ;; since that would overlook other kinds of constants. But it turns
@@ -1089,7 +1113,32 @@
   (if (and (not env)
            (typep spec '(cons (eql quote) (cons t null))))
       (with-current-source-form (spec)
-        (source-transform-typep object (cadr spec)))
+        ;; Decline to do the source transform when seeing an unknown
+        ;; type immediately while block converting, since it may be
+        ;; defined later. By waiting for the deftransform to fire
+        ;; during block compilation, we give ourselves a better chance
+        ;; at open-coding the type test.
+        (let ((type (cadr spec)))
+          ;;
+          #+collect-typep-regression-dataset
+          (let ((parse (specifier-type type)))
+            ;; alien types aren't externalizable as trees of symbols,
+            ;; and some classoid types aren't defined at the start of warm build,
+            ;; making it impossible to re-parse a dump produced late in the build.
+            ;; Luckily there are no cases involving compund types and classoids.
+            (unless (or (involves-alien-p parse)
+                        (or (classoid-p parse)
+                            (and (cons-type-p parse)
+                                 (classoid-p (cons-type-car-type parse)))))
+              (let ((table *interesting-types*))
+                (unless (hash-table-p table)
+                  (setq table (dump/restore-interesting-types 'read)))
+                (setf (gethash type table) t))))
+          ;;
+          (if (and (block-compile *compilation*)
+                   (contains-unknown-type-p (careful-specifier-type type)))
+              (values nil t)
+              (source-transform-typep object type))))
       (values nil t)))
 
 ;;;; coercion
