@@ -576,6 +576,10 @@
   (target nil :type (or null tn-ref))
   ;; the load TN allocated for this operand, if any
   (load-tn nil :type (or tn null))
+  ;; on CISC microprocessors, a representation of the memory load
+  ;; (or store) based on this TN-REF (for which its TN is the base register)
+  ;; and not directly an operand from/into which the TN-REF flows.
+  (memory-access)
   ;; The type of the LVAR the TN of this TN-REF is used for.
   (type nil :type (or ctype null)))
 
@@ -732,6 +736,10 @@
   (targets nil :type (or null (simple-array (unsigned-byte 16) 1)))
   (optimizer nil :type (or null function))
   move-vop-p)
+(declaim (inline vop-name))
+(defun vop-name (vop)
+  (declare (type vop vop))
+  (vop-info-name (vop-info vop)))
 
 ;; These printers follow the definition of VOP-INFO because they
 ;; want to inline VOP-INFO-NAME, and it's less code to move them here
@@ -744,12 +752,8 @@
 (defprinter (tn-ref)
   tn
   write-p
-  (vop :test vop :prin1 (vop-info-name (vop-info vop))))
+  (vop :test vop :prin1 (vop-name vop)))
 
-(declaim (inline vop-name))
-(defun vop-name (vop)
-  (declare (type vop vop))
-  (vop-info-name (vop-info vop)))
 
 ;;;; SBs and SCs
 
@@ -1110,8 +1114,7 @@
             (:copier nil))
   ;; the IR2-BLOCK that this structure represents the conflicts for
   (block (missing-arg) :type ir2-block)
-  ;; thread running through all the GLOBAL-CONFLICTSs for BLOCK. This
-  ;; thread is sorted by TN number
+  ;; thread running through all the GLOBAL-CONFLICTSs for BLOCK.
   (next-blockwise nil :type (or global-conflicts null))
   ;; the way that TN is used by BLOCK
   ;;
